#!/usr/bin/env python
#
# Copyright (c) 2015 mindsensors.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date      Author      Comments
# 10/14/15  Michael     Initial Authoring
# 10/21/15  Michael     ABSIMU heading fix and MMX encoder reset

from mindsensors_i2c import mindsensors_i2c

## @package mindsensors
#  This module contains classes and functions necessary for use of mindsensors.com I2C devices with Raspberry Pi

## ABSIMU: this class provides functions for models of the ABSIMU from mindsensors.com
#  for read and write operations.
class ABSIMU(mindsensors_i2c):  

    ## Default ABSIMU I2C Address 
    ABSIMU_ADDRESS = (0x22)
    ## Command Register 
    COMMAND = 0x41
    ## X-Axis Tilt Register. Will return a signed integer reading
    TILT_X = 0x42
    ## Y-Axis Tilt Register. Will return a signed integer reading
    TILT_Y = 0x43
    ## Z-Axis Tilt Register. Will return a signed integer reading
    TILT_Z = 0x44
    ## X-Axis Accelerometer Register. Will return a signed integer reading (-1050 - 1050) 
    ACCEL_X = 0x45
    ## Y-Axis Accelerometer Register. Will return a signed integer reading (-1050 - 1050)
    ACCEL_Y = 0x47
    ## Z-Axis Accelerometer Register. Will return a signed integer reading (-1050 - 1050)
    ACCEL_Z = 0x49
    ## Compass Heading Register. Will return an unsigned integer reading (0 - 360)
    CMPS = 0x4B
    ## X-Axis Magnetometer Register. Will return a signed integer reading 
    MAG_X = 0x4D
    ## Y-Axis Magnetometer Register. Will return a signed integer reading 
    MAG_Y = 0x4F
    ## Z-Axis Magnetometer Register. Will return a signed integer reading 
    MAG_Z = 0x51
    ## X-Axis Gyroscope Register. Will return a signed integer reading 
    GYRO_X = 0x53
    ## Y-Axis Gyroscope Register. Will return a signed integer reading 
    GYRO_Y = 0x55
    ## Z-Axis Gyroscope Register. Will return a signed integer reading 
    GYRO_Z = 0x57
    
    ## Initialize the class with the i2c address of your AbsoluteIMU
    #  @param self The object pointer.
    #  @param i2c_address Address of your AbsoluteIMU.
    #  @remark
    def __init__(self, absimu_address = ABSIMU_ADDRESS):
        mindsensors_i2c.__init__(self, absimu_address >> 1)        
        
    ## Writes a value to the command register
    #  @param self The object pointer.
    #  @param commands Value to write to the command register.
    def command(self, command):
        self.writeByte(COMMAND, int(cmd)) 
    
    ## Reads the tilt value along the x-axis
    #  @param self The object pointer.
    def get_tiltx(self):
        try:
            return self.readByteSigned(self.TILT_X)
        except:
            print "Error: Could not read tilt along x-axis"
            return ""
      
    ## Reads the tilt value along the y-axis
    #  @param self The object pointer.
    def get_tilty(self):
        try:
            return self.readByteSigned(self.TILT_Y)
        except:
            print "Error: Could not read tilt along y-axis"
            return ""
            
    ## Reads the tilt value along the z-axis
    #  @param self The object pointer.
    def get_tiltz(self):
        try:
            return self.readByteSigned(self.TILT_Z)
        except:
            print "Error: Could not read tilt along z-axis"
            return ""
            
    ## Reads the tilt values 
    #  @param self The object pointer.
    def get_tiltall(self):
        try:
            res = [(self.get_tiltx(),
                    self.get_tilty(),
                    self.get_tiltz())]
            return res
        except:
            print "Error: Could not read tilt values"
            return ""
            
    ## Reads acceleromter value along the x-axis
    #  @param self The object pointer.
    def get_accelx(self):
        try:
            return self.readIntegerSigned(self.ACCEL_X)
        except:
            print "Error: Could not read accelerometer value along x-axis"
            return ""
            
    ## Reads acceleromter value along the y-axis
    #  @param self The object pointer.
    def get_accely(self):
        try:
            return self.readIntegerSigned(self.ACCEL_Y)
        except:
            print "Error: Could not read accelerometer value along y-axis"
            return ""
            
    ## Reads acceleromter value along the z-axis
    #  @param self The object pointer.
    def get_accelz(self):
        try:
            return self.readIntegerSigned(self.ACCEL_Z)
        except:
            print "Error: Could not read accelerometer value along z-axis"
            return ""
            
    ## Reads the accelerometer values 
    #  @param self The object pointer.
    def get_accelall(self):
        try:
            res = [(self.get_accelx(),
                    self.get_accely(),
                    self.get_accelz())]
            return res
        except:
            print "Error: Could not read accelerometer values"
            return ""
            
    ## Reads compass heading
    #  @param self The object pointer.
    def get_heading(self):
        try:
            head = self.readInteger(self.CMPS)
            while(head > 360 or head < 0):
                head = self.readInteger(self.CMPS)
            return head
        except:
            print "Error: Could not read compass heading"
            return ""
    
    ## Reads magnetometer value along the x-axis
    #  @param self The object pointer.
    def get_magx(self):
        try:
            return self.readIntegerSigned(self.MAG_X)
        except:
            print "Error: Could not read magnetometer value along x-axis"
            return ""
            
    ## Reads magnetometer value along the y-axis
    #  @param self The object pointer.
    def get_magy(self):
        try:
            return self.readIntegerSigned(self.MAG_Y)
        except:
            print "Error: Could not read magnetometer value along y-axis"
            return ""
            
    ## Reads magnetometer value along the z-axis
    #  @param self The object pointer.
    def get_magz(self):
        try:
            return self.readIntegerSigned(self.MAG_Z)
        except:
            print "Error: Could not read magnetometer value along z-axis"
            return ""
            
    ## Reads the magnetometer values 
    #  @param self The object pointer.
    def get_magall(self):
        try:
            res = [(self.get_magx(),
                    self.get_magy(),
                    self.get_magz())]
            return res
        except:
            print "Error: Could not read magnetometer values"
            return ""
            
    ## Reads gyroscope value along the x-axis
    #  @param self The object pointer.
    def get_gyrox(self):
        try:
            return self.readIntegerSigned(self.GYRO_X)
        except:
            print "Error: Could not read gyroscope value along x-axis"
            return ""
            
    ## Reads gyroscope value along the y-axis
    #  @param self The object pointer.
    def get_gyroy(self):
        try:
            return self.readIntegerSigned(self.GYRO_Y)
        except:
            print "Error: Could not read gyroscope value along y-axis"
            return ""
            
    ## Reads gyroscope value along the z-axis
    #  @param self The object pointer.
    def get_gyroz(self):
        try:
            return self.readIntegerSigned(self.GYRO_Z)
        except:
            print "Error: Could not read gyroscope value along z-axis"
            return ""
            
    ## Reads the tilt values 
    #  @param self The object pointer.
    def get_gyroall(self):
        try:
            res = [(self.get_gyrox(),
                    self.get_gyroy(),
                    self.get_gyroz())]
            return res
        except:
            print "Error: Could not read gyroscope values"
            return ""
            
    ## Starts the compass calibration process
    #  @param self The object pointer.
    def start_cmpscal(self):
        try:
            self.command(67)
        except:
            print "Error: Could not start compass calibration process"
            return ""    

    ## Stops the compass calibration process
    #  @param self The object pointer.
    def stop_cmpscal(self):
        try:
            self.command(99)
        except:
            print "Error: Could not stop compass calibration process"
            return ""   
    
    ## Sets accelerometer sensitivity to 2G
    #  @param self The object pointer.
    def accel_2G(self):
        try:
            self.command(49)
        except:
            print "Error: Could not change accelerometer sensitivity"
            return ""   

    ## Sets accelerometer sensitivity to 4G
    #  @param self The object pointer.
    def accel_4G(self):
        try:
            self.command(50)
        except:
            print "Error: Could not change accelerometer sensitivity"
            return ""   

    ## Sets accelerometer sensitivity to 8G
    #  @param self The object pointer.
    def accel_8G(self):
        try:
            self.command(51)
        except:
            print "Error: Could not change accelerometer sensitivity"
            return "" 

    ## Sets accelerometer sensitivity to 16G
    #  @param self The object pointer.
    def accel_16G(self):
        try:
            self.command(52)
        except:
            print "Error: Could not change accelerometer sensitivity"
            return ""  

## Angle: this class provides functions for GlideWheel-AngleSensor from mindsensors.com
#  for read and write operations.
class ANGLE(mindsensors_i2c):  
    
    ## Default Angle Sensor I2C Address 
    ANGLE_ADDRESS = (0x30)
    ## Command Register
    COMMAND = 0x41
    ## Angle Register. Will return a long signed integer reading
    ANGLE = 0x42
    ## Raw Angle Register. Will return a long signed integer reading
    RAW = 0x46
    ## RPM Rate Register. Will return a signed integer reading
    RPM = 0x4A
    
    ## Initialize the class with the i2c address of your AngleSensor
    #  @param self The object pointer.
    #  @param i2c_address Address of your AngleSensor.
    def __init__(self, angle_address = ANGLE_ADDRESS):
        #the ANGLE address
        mindsensors_i2c.__init__(self, angle_address >> 1)        
        
    ## Writes a value to the command register
    #  @param self The object pointer.
    #  @param commands Value to write to the command register.
    def command(self, command):
        self.writeByte(self.COMMAND, int(cmd)) 
    
    ## Reads the angle in degrees
    #  @param self The object pointer.
    def get_angle(self):
        try:
            return self.readLongSigned(self.ANGLE)
        except:
            print "Error: Could not read angle"
            return ""
            
    ## Reads the raw angle value, raw = angle x 2
    #  @param self The object pointer.
    def get_raw(self):
        try:
            return self.readLongSigned(self.RAW)
        except:
            print "Error: Could not read raw angle"
            return ""
    
    ## Reads the rotations per minute
    #  @param self The object pointer.
    def get_rpm(self):
        try:
            return self.readIntegerSigned(self.RPM)
        except:
            print "Error: Could not read rpm"
            return "" 
            
## CURRENT: this class provides functions for NXTCurrentMeter from mindsensors.com
#  for read and write operations.
class CURRENT(mindsensors_i2c):  

    ## Default CurrentMeter I2C Address 
    CURRENT_ADDRESS = (0x28)
    ## Command Register
    COMMAND = 0x41
    ## Absolute Calibrated Current value Register. Will Return a signed integer value
    CAL = 0x43
    ## Relative Current value Register. Will Return a signed integer value
    REL = 0x45
    ## Reference Current value Register. Will Return a signed integer value
    REF = 0x47
   
    ## Initialize the class with the i2c address of your NXTCurrentMeter
    #  @param self The object pointer.
    #  @param i2c_address Address of your NXTCurrentMeter.
    def __init__(self, current_address = CURRENT_ADDRESS):
        #the DIST address
        mindsensors_i2c.__init__(self, current_address >> 1)
                
    ## Writes a value to the command register
    #  @param self The object pointer.
    #  @param commands Value to write to the command register.
    def command(self, cmd):
        self.writeByte(self.COMMAND, int(cmd)) 
    
    ## Reads the absolute current value in milliAmps
    #  @param self The object pointer.
    def get_calibrated(self):
        try:
            return self.readIntegerSigned(self.CAL)
        except:
            print "Error: Could not read calibrated current"
            return ""
        
    ## Reads the relative current value in milliAmps
    #  @param self The object pointer.
    def get_relative(self):
        try:
            return self.readIntegerSigned(self.REL)
        except:
            print "Error: Could not read relative current"
            return ""
        
    ## Reads the reference current value in milliAmps
    #  @param self The object pointer.
    def get_reference(self):
        try:
            return self.readIntegerSigned(self.REF)
        except:
            print "Error: Could not read reference current"
            return ""
            
    ## Sets the reference current equal to the absolute current value
    #  @param self The object pointer.
    def set_reference(self):
        try:
            self.command(68)
        except:
            print "Error: Could not set reference current"
            return ""
            
## DIST: this class provides functions for DISTNx from mindsensors.com
#  for read and write operations.
class DIST(mindsensors_i2c):  
   
    ## Default Dist-Nx I2C Address 
    DIST_ADDRESS = (0x02)
    ## Command Register
    COMMAND = 0x41
    ## Distance Register. Will return an integer value
    DISTANCE = 0x42
    ## Voltage Register. Will return an integer value
    VOLTAGE = 0x44
#    TYPE = 0x44
   
    ## Initialize the class with the i2c address of your Dist-Nx
    #  @param self The object pointer.
    #  @param i2c_address Address of your Dist-Nx.
    def __init__(self, dist_address = DIST_ADDRESS):
        #the DIST address
        mindsensors_i2c.__init__(self, dist_address >> 1)
                
    ## Writes a value to the command register
    #  @param self The object pointer.
    #  @param commands Value to write to the command register.
    def command(self, command):
        self.writeByte(self.COMMAND, int(cmd)) 
    
    ## Reads the distance in millimeters
    #  @param self The object pointer.
    def get_distance(self):
        try:
            return self.readInteger(self.DISTANCE)
        except:
            print "Error: Could not read distance"
            return ""
        
    ## Reads the distance in inches
    #  @param self The object pointer.
    def get_distance_inches(self):
        try:
            d = self.get_distance()
            return d/25
        except:
            print "Error: Could not read distance"
            return ""
        
    ## Reads the voltage of the Dist-Nx
    #  @param self The object pointer.
    def get_voltage(self):
        try:
            return self.readInteger(self.VOLTAGE)
        except:
            print "Error: Could not read voltage"
            return ""  
            
## EV3SensAdapt: this class provides functions for EV3 Sensor Adapter and EV3 Sensor Multiplexor from mindsensors.com
#  for read and write operations.
class EV3SensAdapt(mindsensors_i2c):  
   
    ## Default Dist-Nx I2C Address 
    EV3SensAdapt_ADDRESS = (0x32)
    ## Set Mode Register
    SETMODE = 0x52
    ## Data Register 1. Will return byte value corresponding to the connected sensor data
    DATA1 = 0x54
    ## Data Register 2. Will return byte value corresponding to the connected sensor data (depending on sensor and mode)
    DATA2 = 0x55
    ## Data Register 3. Will return byte value corresponding to the connected sensor data (depending on sensor and mode)
    DATA3 = 0x56
    ## Data Register 4. Will return byte value corresponding to the connected sensor data (depending on sensor and mode)
    DATA4 = 0x57
    ## Data Register 5. Will return byte value corresponding to the connected sensor data (only on infrared sensor)
    DATA5 = 0x58
    ## Data Register 6. Will return byte value corresponding to the connected sensor data (only on infrared sensor)
    DATA6 = 0x59
    ## Data Register 7. Will return byte value corresponding to the connected sensor data (only on infrared sensor)
    DATA7 = 0x5A
    ## Data Register 8. Will return byte value corresponding to the connected sensor data (only on infrared sensor)
    DATA8 = 0x5B
   
    ## Initialize the class with the i2c address of your Dist-Nx
    #  @param self The object pointer.
    #  @param i2c_address Address of your Dist-Nx.
    def __init__(self, ev3SensAdapt_address = EV3SensAdapt_ADDRESS):
        #the EV3SensorAdapter or EV3SensorMux channel address
        mindsensors_i2c.__init__(self, ev3SensAdapt_address >> 1)
        
    def setMode(self, mode):
        self.writeByte(self.SETMODE, mode) 
        
    def getMode(self):
        return self.readByte(self.SETMODE) 
                
    ## Reads the status of the EV3 Touch Sensor on the EV3SensorAdapter or EV3SensorMux
    #  @param self The object pointer.
    #  @param commands Value to write to the command register.
    def isTouchedEV3(self):
        try:
            self.setMode(15) 
            if(self.readByte(self.DATA1) == 1):
                return self.readByte(self.DATA1) == 1
        except:
            print "Error: Could not read EV3 Touch Sensor from Adapter"
            return ""
    
    def numTouchesEV3(self):
        try:
            self.setMode(15)
            return self.readByte(self.DATA2)
        except:
            print "Error: Could not read EV3 Touch Sensor from Adapter"
            return ""
        
    def resetTouchesEV3(self):
        try:
            self.setMode(15)
            return self.writeByte(self.DATA2, 0)
        except:
            print "Error: Could not read EV3 Touch Sensor from Adapter"
            return ""
        
    def distanceIREV3(self):
        try:
            self.setMode(0) 
            return self.readByte(self.DATA1)
        except:
            print "Error: Could not read EV3 Infrared Sensor from Adapter"
            return ""
           
    def headingIREV3(self,channel):
        try:
            self.setMode(1) 
            return self.readByteSigned(self.DATA1 + ((channel-1)*2))
        except:
            print "Error: Could not read EV3 Infrared Sensor from Adapter"
            return ""
            
    def distanceRemoteIREV3(self,channel):
        try:
            self.setMode(1) 
            return self.readByte(self.DATA1 + (((channel-1)*2)+1))
        except:
            print "Error: Could not read EV3 Infrared Sensor from Adapter"
            return ""

    def remoteLeft(self,channel):
        try:
            self.setMode(2) 
            remote = self.readByte(self.DATA1 + (channel-1))
            if(remote == 0 or remote == 3 or remote == 4):
                return 0
            if(remote == 1 or remote == 5 or remote == 6):
                return 1
            if(remote == 2 or remote == 7 or remote == 8):
                return -1
        except:
            print "Error: Could not read EV3 Infrared Sensor from Adapter"
            return ""

    def remoteRight(self,channel):
        try:
            self.setMode(2) 
            remote = self.readByte(self.DATA1 + (channel-1))
            if(remote == 0 or remote == 1 or remote == 2):
                return 0
            if(remote == 3 or remote == 7 or remote == 5):
                return 1
            if(remote == 4 or remote == 6 or remote == 8):
                return -1    
        except:
            print "Error: Could not read EV3 Infrared Sensor from Adapter"
            return ""

    def distanceUSEV3(self):
        try:
            self.setMode(0) 
            return self.readInteger(self.DATA1)
        except:
            print "Error: Could not read EV3 Ultrasonic Sensor from Adapter"
            return ""
            
    def distanceUSEV3in(self):
        try:
            self.setMode(1) 
            return self.readInteger(self.DATA1)
        except:
            print "Error: Could not read EV3 Ultrasonic Sensor from Adapter"
            return ""
            
    def presenceUSEV3(self):
        try:
            self.setMode(2) 
            return self.readByte(self.DATA1) == 1
        except:
            print "Error: Could not read EV3 Ultrasonic Sensor from Adapter"
            return ""
        
    def gyroAngleEV3(self):
        try:
            self.setMode(0) 
            return self.readIntegerSigned(self.DATA1)
        except:
            print "Error: Could not read EV3 Gyro Sensor from Adapter"
            return ""
        
    def gyroRateEV3(self):
        try:
            self.setMode(1) 
            return self.readIntegerSigned(self.DATA1)
        except:
            print "Error: Could not read EV3 Gyro Sensor from Adapter"
            return ""
        
    def reflectedLightSensorEV3(self):
        try:
            self.setMode(0) 
            return self.readInteger(self.DATA1)
        except:
            print "Error: Could not read EV3 Color Sensor from Adapter"
            return ""
        
    def ambientLightSensorEV3(self):
        try:
            self.setMode(1) 
            return self.readInteger(self.DATA1)
        except:
            print "Error: Could not read EV3 Color Sensor from Adapter"
            return ""
        
    def colorSensorEV3(self):
        try:
            self.setMode(2) 
            return self.readInteger(self.DATA1)
        except:
            print "Error: Could not read EV3 Color Sensor from Adapter"
            return ""
    
            
## LINELEADER: this class provides functions for Lineleader from mindsensors.com
#  for read and write operations.
class LINELEADER(mindsensors_i2c):

    ## Default Lineleader I2C Address 
    LL_ADDRESS = 0x02
    ## Command Register
    LL_COMMAND = 0x41
    ## Steering Register. Will return a signed byte value
    LL_STEERING = 0x42
    ## Average Register. Will return a byte value
    LL_AVERAGE = 0x43
    ## Steering Register. Will return a byte value
    LL_RESULT = 0x44
    ## Setpoint Register
    LL_SETPOINT = 0x45
    ## KP Register
    LL_Kp = 0x46
    ## Ki Register
    LL_KI = 0x47
    ## Kd Register
    LL_KD = 0x48
    ## Kp factor Register
    LL_KPfactor = 0x61
    ## Ki factor Register
    LL_KIfactor = 0x62
    ## Kd factor Register
    LL_KDfactor = 0x63
    
    LL_CALIBRATED = 0x49
    LL_UNCALIBRATED = 0x74
       
    ## Initialize the class with the i2c address of your Lineleader
    #  @param self The object pointer.
    #  @param i2c_address Address of your LightSensorArray.
    def __init__(self, ll_address = LL_ADDRESS):
        #the LSA address
        mindsensors_i2c.__init__(self, ll_address >> 1)        
        
    ## Writes a value to the command register
    #  @param self The object pointer.
    #  @param commands Value to write to the command register.
    def command(self, cmd):
        self.writeByte(self.LL_COMMAND, int(cmd)) 

    ## Calibrates the white value for the LightSensorArray
    #  @param self The object pointer.
    def White_Cal(self):
        self.command(87)
        
    ## Calibrates the black value for the LightSensorArray
    #  @param self The object pointer.
    def Black_Cal(self):
        self.command(66)
        
    ## Wakes up or turns on the LEDs of the LightSensorArray
    #  @param self The object pointer.
    def Wakeup(self):
        self.command(80)
        
    ## Puts to sleep, or turns off the LEDs of the LightSensorArray
    #  @param self The object pointer.
    def Sleep(self):
        self.command(68)
    
    ## Reads the eight(8) calibrated light sensor values of the LightSensorArray
    #  @param self The object pointer.
    def ReadRaw_Calibrated(self):
        try:
            return self.readArray(self.LL_CALIBRATED, 8)
        except:
            print "Error: Could not read Lineleader"
            return ""
    
    ## Reads the eight(8) uncalibrated light sensor values of the LightSensorArray
    #  @param self The object pointer.    
    def ReadRaw_Uncalibrated(self):
        try:
            s1 = self.readInteger(self.LL_UNCALIBRATED)
            s2 = self.readInteger(self.LL_UNCALIBRATED + 2)
            s3 = self.readInteger(self.LL_UNCALIBRATED + 4)
            s4 = self.readInteger(self.LL_UNCALIBRATED + 6)
            s5 = self.readInteger(self.LL_UNCALIBRATED + 8)
            s6 = self.readInteger(self.LL_UNCALIBRATED + 10)
            s7 = self.readInteger(self.LL_UNCALIBRATED + 12)
            s8 = self.readInteger(self.LL_UNCALIBRATED + 14)
            array = [s1, s2, s3, s4, s5, s6, s7, s8]
            return array
        except:
            print "Error: Could not read Lineleader"
            return ""
     
    ## Read the steering value from the Lineleader (add or subtract this value to the motor speed)
    #  @param self The object pointer.
    def steering(self):
        try:
            return self.readByteSigned(self.LL_STEERING)
        except:
            print "Error: Could not read Lineleader"
            return ""  
    
    ## Read the average weighted value of the current line from position from the Lineleader
    #  @param self The object pointer.
    def average(self):
        try:
            return self.readByte(self.LL_AVERAGE)
        except:
            print "Error: Could not read Lineleader"
            return ""  

    ## Reads the result of all 8 light sensors form the LineLeader as 1 byte (1 bit for each sensor) 
    #  @param self The object pointer.
    def result(self):
        try:
            return self.readByte(self.LL_RESULT)
        except:
            print "Error: Could not read Lineleader"
            return ""       
            
    ## Reads the eight(8) calibrated light sensor values of the LightSensorArray
    #  @param self The object pointer.
    def getSetPoint(self):
        try:
            return self.readByte(self.LL_SETPOINT)
        except:
            print "Error: Could not read Lineleader"
            return ""
            
    ## Reads the eight(8) calibrated light sensor values of the LightSensorArray
    #  @param self The object pointer.
    def setSetPoint(self):
        try:
            return self.writeByte(self.LL_SETPOINT)
        except:
            print "Error: Could not write to Lineleader"
            return ""
            
    ## Write the Kp value to the Lineleader
    #  @param self The object pointer.
    def setKP(self):
        try:
            return self.writeByte(self.LL_KP)
        except:
            print "Error: Could not write to Lineleader"
            return ""
            
    ## Write the Ki value to the Lineleader
    #  @param self The object pointer.
    def setKI(self):
        try:
            return self.writeByte(self.LL_KI)
        except:
            print "Error: Could not write to Lineleader"
            return ""
            
    ## Write the Kd value to the Lineleader
    #  @param self The object pointer.
    def setKD(self):
        try:
            return self.writeByte(self.LL_KD)
        except:
            print "Error: Could not write to Lineleader"
            return ""
            
    ## Write the Kp factor value to the Lineleader
    #  @param self The object pointer.
    def setKPfactor(self):
        try:
            return self.writeByte(self.LL_KPfactor)
        except:
            print "Error: Could not write to Lineleader"
            return ""
            
    ## Write the Ki factor value to the Lineleader
    #  @param self The object pointer.
    def setKIfactor(self):
        try:
            return self.writeByte(self.LL_KIfactor)
        except:
            print "Error: Could not write to Lineleader"
            return ""
            
    ## Write the Kd factor value to the Lineleader
    #  @param self The object pointer.
    def setKDfactor(self):
        try:
            return self.writeByte(self.LL_KDfactor)
        except:
            print "Error: Could not write to Lineleader"
            return ""
    
    ## Read the Kp value from the Lineleader
    #  @param self The object pointer.
    def getKP(self):
        try:
            return self.readByte(self.LL_KP)
        except:
            print "Error: Could not read Lineleader"
            return ""
            
    ## Read the Ki value from the Lineleader
    #  @param self The object pointer.
    def getKI(self):
        try:
            return self.readByte(self.LL_KI)
        except:
            print "Error: Could not read Lineleader"
            return ""
            
    ## Read the Kd value from the Lineleader
    #  @param self The object pointer.
    def getKD(self):
        try:
            return self.readByte(self.LL_KD)
        except:
            print "Error: Could not read Lineleader"
            return ""
            
    ## Read the Kp factor value to the Lineleader
    #  @param self The object pointer.
    def getKPfactor(self):
        try:
            return self.readByte(self.LL_KPfactor)
        except:
            print "Error: Could not read Lineleader"
            return ""
            
    ## Read the Ki factor value to the Lineleader
    #  @param self The object pointer.
    def getKIfactor(self):
        try:
            return self.readByte(self.LL_KIfactor)
        except:
            print "Error: Could not read Lineleader"
            return ""
     
    ## Read the Kd factor value to the Lineleader
    #  @param self The object pointer.
    def getKDfactor(self):
        try:
            return self.readByte(self.LL_KDfactor)
        except:
            print "Error: Could not read Lineleader"
            return ""             

## LSA: this class provides functions for LightSensorArray from mindsensors.com
#  for read and write operations.
class LSA(mindsensors_i2c):  
    
    ## Default LightSensorArray I2C Address 
    LSA_ADDRESS = 0x14
    ## Command Register
    LSA_COMMAND = 0x41
    ## Calibrated Register. Will return an 8 byte array
    LSA_CALIBRATED = 0x42
    ## Uncalibrated Register. Will return an 8 byte array
    LSA_UNCALIBRATED = 0x6A
    
    ## Initialize the class with the i2c address of your LightSensorArray
    #  @param self The object pointer.
    #  @param i2c_address Address of your LightSensorArray.
    def __init__(self, lsa_address = LSA_ADDRESS):
        #the LSA address
        mindsensors_i2c.__init__(self, lsa_address >> 1)        
        
    ## Writes a value to the command register
    #  @param self The object pointer.
    #  @param commands Value to write to the command register.
    def command(self, cmd):
        self.writeByte(self.LSA_COMMAND, int(cmd)) 

    ## Calibrates the white value for the LightSensorArray
    #  @param self The object pointer.
    def White_Cal(self):
        self.command(87)
        
    ## Calibrates the black value for the LightSensorArray
    #  @param self The object pointer.
    def Black_Cal(self):
        self.command(66)
        
    ## Wakes up or turns on the LEDs of the LightSensorArray
    #  @param self The object pointer.
    def Wakeup(self):
        self.command(80)
        
    ## Puts to sleep, or turns off the LEDs of the LightSensorArray
    #  @param self The object pointer.
    def Sleep(self):
        self.command(68)
    
    ## Reads the eight(8) calibrated light sensor values of the LightSensorArray
    #  @param self The object pointer.
    def ReadRaw_Calibrated(self):
        try:
            return self.readArray(self.LSA_CALIBRATED, 8)
        except:
            print "Error: Could not read LSArray"
            return ""
    
    ## Reads the eight(8) uncalibrated light sensor values of the LightSensorArray
    #  @param self The object pointer.    
    def ReadRaw_Uncalibrated(self):
        try:
            s1 = self.readInteger(self.LSA_UNCALIBRATED)
            s2 = self.readInteger(self.LSA_UNCALIBRATED + 2)
            s3 = self.readInteger(self.LSA_UNCALIBRATED + 4)
            s4 = self.readInteger(self.LSA_UNCALIBRATED + 6)
            s5 = self.readInteger(self.LSA_UNCALIBRATED + 8)
            s6 = self.readInteger(self.LSA_UNCALIBRATED + 10)
            s7 = self.readInteger(self.LSA_UNCALIBRATED + 12)
            s8 = self.readInteger(self.LSA_UNCALIBRATED + 14)
            array = [s1, s2, s3, s4, s5, s6, s7, s8]
            return array
        except:
            print "Error: Could not read LSArray"
            return "" 

## MMX: this class provides motor control functions for use with NXTMMX
#  Raspberry Pi i2c baud rate must be 35000 or less to ensure proper functionality of NXTMMX.
class MMX(mindsensors_i2c):
    
    ## Default NXTMMX I2C Address 
    MMX_ADDRESS = (0x06)
    ## Constant Voltage Multipler
    MMX_VOLTAGE_MULTIPLIER = 37    
    
    ## Constant to specify Motor 1
    MMX_Motor_1        =        0x01
    ## Constant to specify Motor 2
    MMX_Motor_2        =        0x02
    ## Constant to specify both Motors
    MMX_Motor_Both     =        0x03

    ## Constant to specify Float Action
    MMX_Next_Action_Float   =   0x00
    ## Constant to specify Brake Action
    MMX_Next_Action_Brake   =   0x01
    ## Constant to specify Hold Action
    MMX_Next_Action_BrakeHold = 0x02
    
    ## Constant to specify Forward Motion
    MMX_Direction_Forward   =   0x01
    ## Constant to specify Reverse Motion
    MMX_Direction_Reverse   =   0x00

    ## Constant to specify Relative Encoder Position
    MMX_Move_Relative = 0x01
    ## Constant to specify Absolute Encoder Position
    MMX_Move_Absolute = 0x00

    ## Constant to Wait for Next Action
    MMX_Completion_Wait_For   =  0x01
    ## Constant to NOT Wait for Next Action
    MMX_Completion_Dont_Wait  = 0x00

    ## Constant for commonly used Full Speed value
    MMX_Speed_Full = 90
    ## Constant for commonly used Moderate Speed value
    MMX_Speed_Medium = 60
    ## Constant for commonly used Slow Speed value
    MMX_Speed_Slow  = 25

    ## Constant to specify Speed bit of Motor Control byte
    MMX_CONTROL_SPEED  =    0x01
    ## Constant to specify Ramp bit of Motor Control byte
    MMX_CONTROL_RAMP   =    0x02
    ## Constant to specify Relative bit of Motor Control byte
    MMX_CONTROL_RELATIVE =  0x04
    ## Constant to specify Tacho bit of Motor Control byte
    MMX_CONTROL_TACHO  =    0x08
    ## Constant to specify Brake bit of Motor Control byte
    MMX_CONTROL_BRK    =    0x10
    ## Constant to specify On bit of Motor Control byte
    MMX_CONTROL_ON     =    0x20
    ## Constant to specify Time bit of Motor Control byte
    MMX_CONTROL_TIME  =     0x40
    ## Constant to specify Go bit of Motor Control byte
    MMX_CONTROL_GO   =      0x80
    
    ## Command Register
    MMX_COMMAND = 0x41
    ## Motor 1 Encoder Target Register
    MMX_SETPT_M1  =   0x42
    ## Motor 1 Speed Register
    MMX_SPEED_M1  =   0x46
    ## Motor 1 Time Register
    MMX_TIME_M1  =    0x47
    ## Motor 1 Motor Command B Register
    MMX_CMD_B_M1  =   0x48
    ## Motor 1 Motor Command A Register
    MMX_CMD_A_M1  =   0x49
    ## Motor 2 Encoder Target Register
    MMX_SETPT_M2  =   0x4A
    ## Motor 2 Speed Register
    MMX_SPEED_M2  =   0x4E
    ## Motor 2 Time Register
    MMX_TIME_M2   =   0x4F
    ## Motor 1 Motor Command B Register
    MMX_CMD_B_M2  =   0x50
    ## Motor 1 Motor Command A Register
    MMX_CMD_A_M2  =   0x51
    ## Motor 1 Encoder Position Register. Will return a long singed integer value
    MMX_POSITION_M1 = 0x62
    ## Motor 2 Encoder Position Register. Will return a long singed integer value
    MMX_POSITION_M2 = 0x66
    ## Motor 1 Status Register. Will return a byte value
    MMX_STATUS_M1   = 0x72
    ## Motor 2 Status Register. Will return a byte value
    MMX_STATUS_M2   = 0x73
    ## Motor 1 Tasks Register. Will return a byte value
    MMX_TASKS_M1    = 0x76
    ## Motor 2 Tasks Register. Will return a byte value
    MMX_TASKS_M2   =  0x77
    ## Position Kp Register
    MMX_P_Kp  =  0x7A   
    ## Position Ki Register    
    MMX_P_Ki  =  0x7C   
    ## Position Kd Register    
    MMX_P_Kd  =  0x7E  
    ## Speed Kp Register    
    MMX_S_Kp  =  0x80  
    ## Speed Ki Register    
    MMX_S_Ki  =  0x82 
    ## Speed Kd Register    
    MMX_S_Kd  =  0x84 
    ## Pass Count Register    
    MMX_PASSCOUNT  =  0x86
    ## Pass Tolerance Register
    MMX_PASSTOLERANCE  =  0x87
    
    ## Initialize the class with the i2c address of your NXTMMX
    #  @param self The object pointer.
    #  @param PiDrive_address Address of your NXTMMX.
    def __init__(self, mmx_address = MMX_ADDRESS):
        #the NXTMMX address
        mindsensors_i2c.__init__(self, mmx_address >> 1)       
    
    ## Writes a specified command on the command register of the NXTMMX
    #  @param self The object pointer.
    #  @param cmd The command you wish the NXTMMX to execute.    
    def command(self, cmd):
       self.writeByte(self.MMX_COMMAND, int(cmd))
       
    ## Reset the both motor encoders of the NXTMMX
    #  @param self The object pointer.
    #  @param motor_number Number of the motor you wish to read. 
    def resPos(self):
       self.command(82)
       
    ## Reads the battery voltage
    #  @param self The object pointer.
    def battVoltage(self):
        try:
            return self.readByte(self.MMX_COMMAND) * self.MMX_VOLTAGE_MULTIPLIER #37
        except:
            print "Error: Could not read voltage"
            return ""
        
    ## Reads the encoder position of the specified motor
    #  @param self The object pointer.
    #  @param motor_number Number of the motor you wish to read.
    def pos(self, motor_number):
        try:
            if motor_number == 1 :
                return self.readLongSigned(self.MMX_POSITION_M1) 
            if motor_number == 2 :
                return self.readLongSigned(self.MMX_POSITION_M2) 
        except:
            print "Error: Could not read encoder position"
            return ""
            
    ## Run the motor(s) at a set speed for an unlimited duration
    #  @param self The object pointer.
    #  @param motor_number Number of the motor(s) you wish to turn.
    #  @param speed The speed at which you wish to turn the motor(s).
    def setSpeed( self, motor_number, speed ):

        ctrl = 0
        ctrl |= self.MMX_CONTROL_SPEED
        ctrl |= self.MMX_CONTROL_BRK
        
        if ( motor_number != self.MMX_Motor_Both ):
            ctrl |= self.MMX_CONTROL_GO
        if ( (motor_number & 0x01) != 0 ): 
            array = [speed, 0, 0, ctrl]
            self.writeArray( self.MMX_SPEED_M1, array)
        if ( (motor_number & 0x02) != 0 ):
            array = [speed, 0, 0, ctrl]
            self.writeArray( self.MMX_SPEED_M2, array); 
        if ( motor_number == self.MMX_Motor_Both ) :
            self.writeByte(self.MMX_COMMAND, 83)  
    
    ### @cond
    ## Stops the specified motor(s)
    #  @param self The object pointer.
    #  @param motor_number Number of the motor(s) you wish to turn.
    #  @param next_action How you wish to stop the motor(s).        
    def MMX_Stop( self, motor_number, next_action ):
        
        if ( next_action == self.MMX_Next_Action_Brake or next_action == self.MMX_Next_Action_BrakeHold ):
            if (motor_number == self.MMX_Motor_1):
                self.writeByte(self.MMX_COMMAND, 65)
            if (motor_number == self.MMX_Motor_2):
                self.writeByte(self.MMX_COMMAND, 66)
            if (motor_number == self.MMX_Motor_Both):
                self.writeByte(self.MMX_COMMAND, 67)
        else:
            if (motor_number == self.MMX_Motor_1):
                self.writeByte(self.MMX_COMMAND, 97)
            if (motor_number == self.MMX_Motor_2):
                self.writeByte(self.MMX_COMMAND, 98)
            if (motor_number == self.MMX_Motor_Both):
                self.writeByte(self.MMX_COMMAND, 99)
                
    def status(self, motor_number):
        if (motor_number == 1):
            return self.readByte(self.MMX_STATUS_M1)
        if (motor_number == 2):
            return self.readByte(self.MMX_STATUS_M2)
    
    def statusBit(self, motor_number, bitno = 0):
        return (self.status(motor_number) >> bitno) & 1
    ### @endcond
                
    ## Stop the motor with abruptly with brake
    #  @param self The object pointer.
    #  @param motor_number Number of the motor(s) you wish to turn.
    def brake(self, motor_number):
        self.MMX_Stop(motor_number, self.MMX_Next_Action_Brake)
            
    ## Stop the motor smoothly with float
    #  @param self The object pointer. 
    #  @param motor_number Number of the motor(s) you wish to turn.
    def float(self, motor_number):
        self.MMX_Stop(motor_number, self.MMX_Next_Action_Float)
        
    ## Stop the motor abruptly and hold the current position
    #  @param self The object pointer. 
    #  @param motor_number Number of the motor(s) you wish to turn.
    def hold(self, motor_number):
        self.MMX_Stop(motor_number, self.MMX_Next_Action_BrakeHold)
        
    ## Check if the motor is running
    #  @param self The object pointer.
    def isBusy(self, motor_number):
        return self.statusBit(motor_number, 0) == 1 or self.statusBit(motor_number, 1) == 1 or self.statusBit(motor_number, 3) == 1 or self.statusBit(motor_number, 6) == 1
    
    ## Wait until the motor is no longer running
    #  @param self The object pointer.
    #  @param timeout The timeout value as a factor of 10ms.
    def waitUntilNotBusy(self, motor_number, timeout=-1):
        while(self.isBusy(motor_number)):
            time.sleep(.01)
            timeout -= 1
            if(timeout == 0):
                return 1
            if(timeout <-5):
                timeout = -1
            pass
        return 0
    
    ## Check if the motor is stalled
    #  @param self The object pointer.
    def isStalled(self, motor_number):
        return self.statusBit(motor_number, 7) == 1
    
    ## Check if the motor is overloaded
    #  @param self The object pointer.  
    def isOverloaded(self):
        return self.statusBit(motor_number, 5) == 1
    
    ## Run the motor for a specific time in seconds
    #  @param self The object pointer.
    #  @param motor_number Number of the motor(s) you wish to turn.
    #  @param secs The number of seconds to run the motor.
    #  @param speed The speed at which to turn the motor.
    #  @param brakeOnCompletion Choose to brake or float the motor upon completion with True (brake) or False (float). 
    #  @param waitForCompletion Wait until the motor is finished running before continuing the program.    
    def runSecs( self, motor_number, secs, speed, brakeOnCompletion = False, waitForCompletion = False ):        
        ctrl = 0
        ctrl |= self.MMX_CONTROL_SPEED
        ctrl |= self.MMX_CONTROL_TIME

        if ( brakeOnCompletion == True ):
            ctrl |= self.MMX_CONTROL_BRK
        if ( motor_number != self.MMX_Motor_Both ):
            ctrl |= self.MMX_CONTROL_GO
        if ( (motor_number & 0x01) != 0 ):
            array = [speed, secs, 0, ctrl]
            self.writeArray( self.MMX_SPEED_M1, array) 
        if ( (motor_number & 0x02) != 0 ) :
            array = [speed, secs, 0, ctrl]
            self.writeArray( self.MMX_SPEED_M2, array)
        if ( motor_number == self.MMX_Motor_Both ) :
            self.writeByte(self.MMX_COMMAND, 83)        
        if ( waitForCompletion == True ):
            time.sleep(0.050);  # this delay is required for the status byte to be available for reading.
            self.MMX_WaitUntilTimeDone(motor_number)
    
    ### @cond
    ## Waits until the specified time for the motor(s) to run is completed
    #  @param self The object pointer.
    #  @param motor_number Number of the motor(s) to wait for.
    def MMX_WaitUntilTimeDone(self,motor_number):
        while self.MMX_IsTimeDone(motor_number) != True:
            time.sleep(0.050)        
        
    ## Checks to ensure the specified time for the motor(s) to run is completed.
    #  @param self The object pointer.
    #  @param motor_number Number of the motor(s) to check.
    def  MMX_IsTimeDone(self, motor_number):
        if ( motor_number == self.MMX_Motor_1 ):
            result = self.readByte(self.MMX_STATUS_M1)
            # look for the time bit to be zero.
            if (( result & 0x40 ) == 0 ):
                return True         
        elif ( motor_number == self.MMX_Motor_2 ) :
            result = self.readByte(self.MMX_STATUS_M2)
            # look for the time bit to be zero.
            if (( result & 0x40 ) == 0 ):
                return True
        elif ( motor_number == self.MMX_Motor_Both ):
            result = self.readByte(self.MMX_STATUS_M1)
            result2 = self.readByte(self.MMX_STATUS_M2)
            # look for both time bits to be zero
            if (((result & 0x40) == 0) &((result2 & 0x40) == 0) ):
                return True
        else :
            return False
    ### @endcond
            
    ## Run the motor for a specific amount of degrees
    #  @param self The object pointer.
    #  @param motor_number Number of the motor(s) you wish to turn.
    #  @param degs The number of degrees to run the motor(s).
    #  @param speed The speed at which to turn the motor(s).
    #  @param brakeOnCompletion Choose to brake or float the motor upon completion with True (brake) or False (float).
    #  @param holdOnCompletion Choose to hold the motor position upon completion with True (hold) or False (release).
    #  @param waitForCompletion Tells the program when to handle the next line of code.
    def runDegs(self, motor_number, degs, speed, brakeOnCompletion = False, holdOnCompletion = False, waitForCompletion = False):
        ctrl = 0
        ctrl |= self.MMX_CONTROL_SPEED
        ctrl |= self.MMX_CONTROL_TACHO
        ctrl |= self.MMX_CONTROL_RELATIVE
        
        d = degs
        t4 = (d/0x1000000)
        t3 = ((d%0x1000000)/0x10000)
        t2 = (((d%0x1000000)%0x10000)/0x100)
        t1 = (((d%0x1000000)%0x10000)%0x100)
        
        if ( brakeOnCompletion == True ):
            ctrl |= self.MMX_CONTROL_BRK
        if ( holdOnCompletion == True ):
            ctrl |= self.MMX_CONTROL_BRK
            ctrl |= self.MMX_CONTROL_ON
        if ( motor_number != self.MMX_Motor_Both ):
            ctrl |= self.MMX_CONTROL_GO        
        if ( (motor_number & 0x01) != 0 ):
            array = [t1, t2, t3, t4, speed, 0, 0, ctrl]
            self.writeArray(self.MMX_SETPT_M1, array) 
        if ( (motor_number & 0x02) != 0 ) :
            array = [t1, t2, t3, t4, speed, 0, 0, ctrl]
            self.writeArray(self.MMX_SETPT_M2, array) 
        if ( motor_number == self.MMX_Motor_Both ) :
            self.writeByte(self.MMX_COMMAND, 83)         
        if ( waitForCompletion == True ):
            time.sleep(0.050);  # this delay is required for the status byte to be available for reading.
            self.MMX_WaitUntilTachoDone(motor_number)
            
    ## Run the motor for a specific amount of rotations
    #  @param self The object pointer.
    #  @param motor_number Number of the motor(s) you wish to turn.
    #  @param rotations The number of rotations to run the motor(s).
    #  @param speed The speed at which to turn the motor(s).
    #  @param brakeOnCompletion Choose to brake or float the motor upon completion with True (brake) or False (float).
    #  @param holdOnCompletion Choose to hold the motor position upon completion with True (hold) or False (release).
    #  @param waitForCompletion Tells the program when to handle the next line of code.
    def runRotations(self, motor_number, rotations, speed, brakeOnCompletion = False, holdOnCompletion = False, waitForCompletion = False):
        ctrl = 0
        ctrl |= self.MMX_CONTROL_SPEED
        ctrl |= self.MMX_CONTROL_TACHO
        ctrl |= self.MMX_CONTROL_RELATIVE
        
        d = rotations * 360
        
        t4 = (d/0x1000000)
        t3 = ((d%0x1000000)/0x10000)
        t2 = (((d%0x1000000)%0x10000)/0x100)
        t1 = (((d%0x1000000)%0x10000)%0x100)
        
        if ( brakeOnCompletion == True ):
            ctrl |= self.MMX_CONTROL_BRK
        if ( holdOnCompletion == True ):
            ctrl |= self.MMX_CONTROL_BRK
            ctrl |= self.MMX_CONTROL_ON
        if ( motor_number != self.MMX_Motor_Both ):
            ctrl |= self.MMX_CONTROL_GO
        if ( (motor_number & 0x01) != 0 ):
            array = [t1, t2, t3, t4, speed, 0, 0, ctrl]
            self.writeArray(self.MMX_SETPT_M1, array) 
        if ( (motor_number & 0x02) != 0 ) :
            array = [t1, t2, t3, t4, speed, 0, 0, ctrl]
            self.writeArray(self.MMX_SETPT_M2, array) 
        if ( motor_number == self.MMX_Motor_Both ) :
            self.writeByte(self.MMX_COMMAND, 83)        
        if ( waitForCompletion == True ):
            time.sleep(0.050);  # this delay is required for the status byte to be available for reading.
            self.MMX_WaitUntilTachoDone(motor_number)

    ## Run the motor for a specific amount of rotations
    #  @param self The object pointer.
    #  @param motor_number Number of the motor(s) you wish to turn.
    #  @param pos The encoder value to which to run the motor(s).
    #  @param speed The speed at which to turn the motor(s).
    #  @param brakeOnCompletion Choose to brake or float the motor upon completion with True (brake) or False (float).
    #  @param holdOnCompletion Choose to hold the motor position upon completion with True (hold) or False (release).
    #  @param waitForCompletion Tells the program when to handle the next line of code.
    def runEncoderPos(self, motor_number, pos, speed, brakeOnCompletion = False, holdOnCompletion = False, waitForCompletion = False):
        ctrl = 0
        ctrl |= self.MMX_CONTROL_SPEED
        ctrl |= self.MMX_CONTROL_TACHO
        d = pos
        
        t4 = (d/0x1000000)
        t3 = ((d%0x1000000)/0x10000)
        t2 = (((d%0x1000000)%0x10000)/0x100)
        t1 = (((d%0x1000000)%0x10000)%0x100)
        
        if ( brakeOnCompletion == True ):
            ctrl |= self.MMX_CONTROL_BRK
        if ( holdOnCompletion == True ):
            ctrl |= self.MMX_CONTROL_BRK
            ctrl |= self.MMX_CONTROL_ON
        if ( motor_number != self.MMX_Motor_Both ):
            ctrl |= self.MMX_CONTROL_GO
        if ( (motor_number & 0x01) != 0 ):
            array = [t1, t2, t3, t4, speed, 0, 0, ctrl]
            self.writeArray(self.MMX_SETPT_M1, array) 
        if ( (motor_number & 0x02) != 0 ):
            array = [t1, t2, t3, t4, speed, 0, 0, ctrl]
            self.writeArray(self.MMX_SETPT_M2, array) 
        if ( motor_number == self.MMX_Motor_Both ) :
            self.writeByte(self.MMX_COMMAND, 83)         
        if ( waitForCompletion == True ):
            time.sleep(0.050);  # this delay is required for the status byte to be available for reading.
            self.MMX_WaitUntilTachoDone(motor_number) 
    
    ### @cond
    ## Waits until the specified tacheomter count for the motor(s) to run is reached.
    #  @param self The object pointer.
    #  @param motor_number Number of the motor(s) to wait for.
    def MMX_WaitUntilTachoDone(self,motor_number):
        while self.MMX_IsTachoDone(motor_number) != True:
            time.sleep(0.050)        
        
    ## Checks to ensure the specified tacheomter count for the motor(s) to run is reached.
    #  @param self The object pointer.
    #  @param motor_number Number of the motor(s) to check. 
    def  MMX_IsTachoDone(self, motor_number):
        if ( motor_number == self.MMX_Motor_1 ):
            result = self.readByte(self.MMX_STATUS_M1)
            # look for the time bit to be zero.
            if (( result & 0x08 ) == 0 ):
                return True         
        elif ( motor_number == self.MMX_Motor_2 ) :
            result = self.readByte(self.MMX_STATUS_M2)
            # look for the time bit to be zero.
            if (( result & 0x08 ) == 0 ):
                return True
        elif ( motor_number == self.MMX_Motor_Both ):
            result = self.readByte(self.MMX_STATUS_M1)
            result2 = self.readByte(self.MMX_STATUS_M2)
            # look for both time bits to be zero
            if (((result & 0x08) == 0) & ((result2 & 0x08) == 0) ):
                return True
        else :
            return False
    ### @endcond
            
    ## Writes user specified values to the PID control registers
    #  @param self The object pointer.
    #  @param Kp_tacho Proportional-gain of the encoder position of the motor.
    #  @param Ki_tacho Integral-gain of the encoder position of the motor.
    #  @param Kd_tacho Derivative-gain of the encoder position of the motor.
    #  @param Kp_speed Proportional-gain of the speed of the motor.
    #  @param Ki_speed Integral-gain of the speed of the motor.
    #  @param Kd_speed Derivative-gain of the speed of the motor.
    #  @param passcount The number of times the encoder reading should be within tolerance.
    #  @param tolerance The tolerance (in ticks) for encoder positioning .
    def SetPerformanceParameters(self, Kp_tacho, Ki_tacho, Kd_tacho, Kp_speed, Ki_speed, Kd_speed, passcount, tolerance):        
        Kp_t1 = Kp_tacho%0x100
        Kp_t2 = Kp_tacho/0x100      
        Ki_t1 = Ki_tacho%0x100
        Ki_t2 = Ki_tacho/0x100
        Kd_t1 = Kd_tacho%0x100
        Kd_t2 = Kd_tacho/0x100
        Kp_s1 = Kp_speed%0x100        
        Kp_s2 = Kp_speed/0x100
        Ki_s1 = Ki_speed%0x100 
        Ki_s2 = Ki_speed/0x100
        Kd_s1 = Kd_speed%0x100
        Kd_s2 = Kd_speed/0x100
        print "Kp_t1: " + str(Kp_t1)
        print "Kp_t2: " + str(Kp_t2)
        print "Ki_t1: " + str(Ki_t1)
        print "Ki_t2: " + str(Ki_t2)
        print "Kd_t1: " + str(Kd_t1)
        print "Kd_t2: " + str(Kd_t2)
        print "Kp_s1: " + str(Kp_s1)
        print "Kp_s2: " + str(Kp_s2)
        print "Ki_s1: " + str(Ki_s1)
        print "Ki_s2: " + str(Ki_s2)
        print "Kd_s1: " + str(Kd_s1)
        print "Kd_s2: " + str(Kd_s2)
        passcount = passcount
        tolerance = tolerance
        array = [Kp_t1 , Kp_t2 , Ki_t1, Ki_t2, Kd_t1, Kd_t2, Kp_s1, Kp_s2, Ki_s1, Ki_s2, Kd_s1, Kd_s2, passcount, tolerance]
        self.writeArray(self.MMX_P_Kp, array)
        
    ## Reads the values of the PID control registers
    #  @param self The object pointer.
    def ReadPerformanceParameters(self):    
        try:
            print "Pkp: " + str(self.readInteger(self.MMX_P_Kp))
            print "Pki: " + str(self.readInteger(self.MMX_P_Ki))
            print "Pkd: " + str(self.readInteger(self.MMX_P_Kd))
            print "Skp: " + str(self.readInteger(self.MMX_S_Kp))
            print "Ski: " + str(self.readInteger(self.MMX_S_Ki))
            print "Skd: " + str(self.readInteger(self.MMX_S_Kd))
            print "Passcount: " + str(self.MMX_PASSCOUNT)
            print "Tolerance: " + str(self.MMX_PASSTOLERANCE)
        except:
            print "Error: Could not read PID values"
            return ""            
            
            
## BLOB: this class is a subclass of NXTCAM. There is no need to call this class directly.
class BLOB():

    ## Initialize the class with the parameters passed from getBlobs() in the NXTCAM() class
    #  @param self The object pointer.
    #  @param color The color of the specified tracked object.
    #  @param left The left coordinate of the specified tracked object.
    #  @param top The top coordinate of the specified tracked object.
    #  @param right The right coordinate of the specified tracked object.
    #  @param bottom The bottom coordinate of the specified tracked object.
    #  @remark
    def __init__(self, color, left, top, right, bottom):
        self.color = color
        self.left = left
        self.top = top
        self.right = right
        self.bottom = bottom        

## NXTCAM: this class provides functions for models of the NXTCAM and PixyAdapter from mindsensors.com
#  for read and write operations.
class NXTCAM(mindsensors_i2c):  

    ## Default NXTCAM I2C Address 
    NXTCAM_ADDRESS = (0x02)
    ## Command Register 
    COMMAND = 0x41
    ## Number of Tracked Ojects Register. Will return a byte (0-8)
    NumberObjects = 0x42
    ## First Register Containing Tracked Object Data. This is to be read in an array
    Color = 0x43
    ## X-axis Top Register
    X_Top = 0x44
    ## Y-axis Top Register
    Y_Top = 0x45
    ## X-axis Bottom Register
    X_Bottom = 0x46
    ## Y-axis Bottom Register
    Y_Bottom = 0x47
    
    ## Initialize the class with the i2c address of your NXTCAM
    #  @param self The object pointer.
    #  @param i2c_address Address of your AngleSensor.
    #  @remark
    def __init__(self, nxtcam_address = NXTCAM_ADDRESS):
        mindsensors_i2c.__init__(self, nxtcam_address >> 1)        
        
    ## Writes a value to the command register
    #  @param self The object pointer.
    #  @param commands Value to write to the command register.
    def command(self, command):
        self.writeByte(COMMAND, int(cmd)) 
    
    ## Sort the detected objects by size
    #  @param self The object pointer.
    def sortSize(self):
        try:
            self.command(65)
        except:
            print "Error: Could write command to Cam."
            return ""
            
    ## Track detected colors as objects
    #  @param self The object pointer.
    def trackObject(self):
        try:
            self.command(66)
        except:
            print "Error: Could write command to Cam."
            return ""
            
    ## Write to image sensor registers
    #  @param self The object pointer.
    def writeImageRegisters(self):
        try:
            self.command(67)
        except:
            print "Error: Could write command to Cam."
            return ""
            
    ## Disable tracking
    #  @param self The object pointer.
    def stopTracking(self):
        try:
            self.command(68)
        except:
            print "Error: Could write command to Cam."
            return ""
            
    ## Enable tracking
    #  @param self The object pointer.
    def startTracking(self):
        try:
            self.command(69)
        except:
            print "Error: Could write command to Cam."
            return ""
       
    ## Get the color map from NXTCAM
    #  @param self The object pointer.
    def getColorMap(self):
        try:
            self.command(71)
        except:
            print "Error: Could write command to Cam."
            return ""
            
##    ## Turn on illumination
    #  @param self The object pointer.
    def illuminationOn(self):
        try:
            self.command(73)
        except:
            print "Error: Could write command to Cam."
            return ""
            
    ## Read to image sensor registers
    #  @param self The object pointer.
    def readImageRegisters(self):
        try:
            self.command(72)
        except:
            print "Error: Could write command to Cam."
            return ""
            
    ## Track detected colors as lines
    #  @param self The object pointer.
    def trackLine(self):
        try:
            self.command(76)
        except:
            print "Error: Could write command to Cam."
            return ""
            
    ## Ping the NXTCAM
    #  @param self The object pointer.
    def ping(self):
        try:
            self.command(80)
        except:
            print "Error: Could write command to Cam."
            return ""
            
    ## Reset the NXTCAM
    #  @param self The object pointer.
    def reset(self):
        try:
            self.command(82)
        except:
            print "Error: Could write command to Cam."
            return ""
      
    ## Send ColorMap to NXTCAM
    #  @param self The object pointer.
    def sendColorMap(self):
        try:
            self.command(83)
        except:
            print "Error: Could write command to Cam."
            return ""
            
    ## Turn off illumination
    #  @param self The object pointer.
    def illuminationOff(self):
        try:
            self.command(84)
        except:
            print "Error: Could write command to Cam."
            return ""
            
    ## Sort tracked objects by color
    #  @param self The object pointer.
    def sortColor(self):
        try:
            self.command(85)
        except:
            print "Error: Could write command to Cam."
            return ""
            
    ## Get the firmware version of the NXTCAM
    #  @param self The object pointer.
    def firmware(self):
        try:
            self.command(86)
        except:
            print "Error: Could write command to Cam."
            return ""
            
    ## Do not sort tracked objects
    #  @param self The object pointer.
    def sortNone(self):
        try:
            self.command(88)
        except:
            print "Error: Could write command to Cam."
            return ""
            
    ## Read the number of objects detected (0-8)
    #  @param self The object pointer.
    def getNumberObjects(self):
        try:
            return self.readByte(self.NumberObjects)
        except:
            print "Error: Could not read from Cam."
            return ""
    
    ## Reads data of the tracked object(s)
    #  @param self The object pointer.
    #  @param blobNum The number of the tracked object.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from mindsensors  import NXTCAM 
    #  ...
    #  cam = NXTCAM()
    #  cam.startTracking()
    #  cam.trackObject() 
    #  b = cam.getBlobs(1) 
    #  print "Color: " + str(b.color)
    #  print "Left: " + str(b.left)
    #  print "Top: " + str(b.top)
    #  print "Right: " + str(b.right)
    #  print "Bottom: " + str(b.bottom)   
    def getBlobs(self, blobNum = 1):
        try:
            
            data= [0,0,0,0,0]
            blobs = self.getNumberObjects()            
            i = blobNum - 1
            if (blobNum > blobs):
                print "blobNum is greater than amount of blobs tracked."
                return 0
            else:            
                #while(i < blobs):              
                data[0] = color = self.readByte(self.Color + (i*5))  
                data[1] = left = self.readByte(self.X_Top + (i*5))
                data[2] = top = self.readByte(self.Y_Top + (i*5))
                data[3] = right = self.readByte(self.X_Bottom + (i*5))
                data[4] = bottom = self.readByte(self.Y_Bottom + (i*5))       
                return BLOB(color,left,top,right,bottom)
        except:
            print "Error: Could not read from Cam."
            return ""
            

## NXTSERVO: this class provides servo motor control functions
class NXTSERVO(mindsensors_i2c):
    
    ## Default NXTServo I2C Address 
    NXTSERVO_ADDRESS = (0xB0)    
    ## Constant Coltage Multiplier
    NXTSERVO_VBATT_SCALER = 41    
    ## Command Register
    NXTSERVO_COMMAND = 0x41   
    ## Input Power Voltage Register. Will Return a signed integer value    
    NXTSERVO_VBATT = 0x62
    
    ## Initialize the class with the i2c address of your NXTServo
    #  @param self The object pointer.
    #  @param PiDrive_address Address of your NXTServo.
    def __init__(self, nxtservo_address = NXTSERVO_ADDRESS):
        mindsensors_i2c.__init__(self, nxtservo_address >> 1)       
    
    ## Writes a specified command on the command register of the NXTServo
    #  @param self The object pointer.
    #  @param cmd The command you wish the NXTServo to execute.    
    def command(self, cmd):
       self.writeByte(self.NXTSERVO_COMMAND, int(cmd))       

    ## Reads NXTServo battery voltage in millivolts
    #  @param self The object pointer.  
    def battVoltage(self):
        try:
            return self.readByte(self.NXTSERVO_VBATT) * self.NXTSERVO_VBATT_SCALER
        except:
            print "Error: Could not read battery voltage"
            return ""
            
    ## Sets the position of a user defined servo
    #  @param self The object pointer
    #  @param servoNumber The number of the servo you wish to move.
    #  @param position The position to set the servo.
    def setPosition(self, servoNumber, position):
        reg = 64 + (servoNumber*2) 
        p1 = position%256
        p2 = position/256
        array = [p1, p2]
        self.writeArray(reg, array)

    ## Sets the default neutral position of a user defined servo
    #  @param self The object pointer
    #  @param servoNumber The number of the servo you wish to set to the default position.
    def setNeutral(self, servoNumber):
        self.command(73)
        time.sleep(.1)
        servo = servoNumber + 48
        self.command(servo) 

## PFMATE: this class provides motor control functions
class PFMATE(mindsensors_i2c):
    
    ## Default PFMate I2C Address
    PFMATE_ADDRESS = (0x48)
    
    ## Constants to specify Float Action
    PFMATE_FLOAT = 0
    ## Constants to specify Forward Motion
    PFMATE_FORWARD = 1
    ## Constants to specify Reverse Motion
    PFMATE_REVERSE = 2
    ## Constants to specify Brake Action
    PFMATE_BRAKE = 3    
    ## Constants to specify Channel 1
    PFMATE_CHANNEL1 = 1
    ## Constants to specify Channel 2
    PFMATE_CHANNEL2 = 2
    ## Constants to specify Channel 3
    PFMATE_CHANNEL3 = 3
    ## Constants to specify Channel 4
    PFMATE_CHANNEL4 = 4
    
    ## Command Register
    PFMATE_COMMAND  = 0x41
    ## Channel Registewr
    PFMATE_CHANNEL  =   0x42   
    ## Motor Define Register
    PFMATE_MOTORS    =   0x43   
    ## Motor Operation A Register
    PFMATE_OPER_A =    0x44     
    ## Motor Speed A Register
    PFMATE_SPEED_A   =   0x45   
    ## Motor Operation B Register
    PFMATE_OPER_B =    0x46     
    ## Motor Speed B Register
    PFMATE_SPEED_B   =   0x47  
    
    ## Initialize the class with the i2c address of your PFMate
    #  @param self The object pointer.
    #  @param PiDrive_address Address of your PFMate.
    def __init__(self, pfmate_address = PFMATE_ADDRESS):
        #the NXTMMX address
        mindsensors_i2c.__init__(self, pfmate_address >> 1)       
    
    ## Writes a specified command on the command register of the PFMate
    #  @param self The object pointer.
    #  @param cmd The command you wish the PFMate to execute.    
    def command(self, cmd):
       self.writeByte(self.PFMATE_COMMAND, int(cmd))       

    ## Controls both motors
    #  @param self The object pointer.
    #  @param channel Communication channel to transmit to LEGO PF IR receiver (1-4).
    #  @param operationA Operation command for motor A (0 = Float, 1 = Forward, 2 = Reverse, 3 = Brake).     
    #  @param speedA Speed of motor A (0-7).    
    #  @param operationB Operation command for motor B (0 = Float, 1 = Forward, 2 = Reverse, 3 = Brake).     
    #  @param speedB Speed of motor B (0-7).    
    def controlBothMotors(self, channel, operationA, speedA, operationB, speedB):
        array = [channel, 0x00, operationA, speedA, operationB, speedB]
        self.writeArray(self.PFMATE_CHANNEL, array)
        time.sleep(.1)
        self.command(71)
        time.sleep(.1)
        
    ## Controls motor A
    #  @param self The object pointer.
    #  @param channel Communication channel to transmit to LEGO PF IR receiver (1-4).
    #  @param operationA Operation command for motor A (0 = Float, 1 = Forward, 2 = Reverse, 3 = Brake).     
    #  @param speedA Speed of motor A (0-7).       
    def controlMotorA(self, channel, operationA, speedA):
        array = [channel, 0x01, operationA, speedA]
        self.writeArray(self.PFMATE_CHANNEL, array)
        time.sleep(.1)
        self.command(71)
        time.sleep(.1)
        
    ## Controls motor B
    #  @param self The object pointer.
    #  @param channel Communication channel to transmit to LEGO PF IR receiver (1-4).
    #  @param operationB Operation command for motor B (0 = Float, 1 = Forward, 2 = Reverse, 3 = Brake).     
    #  @param speedB Speed of motor B (0-7).    
    def controlMotorB(self, channel, operationB, speedB):
        array = [channel, 0x02]
        array2 = [operationB, speedB]
        self.writeArray(self.PFMATE_CHANNEL, array)
        self.writeArray(self.PFMATE_OPER_B, array2)
        time.sleep(.1)
        self.command(71)
        time.sleep(.1)        

## PPS58: this class provides functions for PPS58 from mindsensors.com
#  for read and write operations.
class PPS58(mindsensors_i2c):  
    
    ## Default PPS58 I2C Address
    PPS58_ADDRESS = (0x18)
    ## Command Register
    COMMAND = 0x41
    ## Unit Register. Will Return a byte value
    UNIT = 0x42
    ## Absolute Pressure Register. Will return an integer value
    ABS = 0x43
    ## Gauge (Relative) Pressure Register. Will return an integer value
    GAUGE = 0x45
    ## Reference Pressure Register. Will return an integer value
    REF = 0x47
    ## Raw Pressure Register. Will return a long integer value
    RAW = 0x53
   
    ## Initialize the class with the i2c address of your PPS58
    #  @param self The object pointer.
    #  @param i2c_address Address of your PPS58.
    def __init__(self, pps58_address = PPS58_ADDRESS):
        #the DIST address
        mindsensors_i2c.__init__(self, pps58_address >> 1)
                
    ## Writes a value to the command register
    #  @param self The object pointer.
    #  @param commands Value to write to the command register.
    def command(self, cmd):
        self.writeByte(self.COMMAND, int(cmd)) 
    
    ## Reads a value assigned to the unit of pressure cureently in use on the PPS58
    #  @param self The object pointer.
    def get_unit(self):
        try:
            return self.readByte(self.UNIT)
        except:
            print "Error: Could not read unit value"
            return ""
    
    ## Reads the absolute pressure of the PPS58
    #  @param self The object pointer.
    def get_absolute(self):
        try:
            return self.readIntegerSigned(self.ABS)
        except:
            print "Error: Could not read absolute pressure"
            return ""
        
    ## Reads the gauge, or relative, pressure of the PPS58
    #  @param self The object pointer.
    def get_gauge(self):
        try:
            return self.readIntegerSigned(self.GAUGE)
        except:
            print "Error: Could not read gauge pressure"
            return ""
        
    ## Reads the reference pressure of the PPS58
    #  @param self The object pointer.
    def get_reference(self):
        try:
            return self.readIntegerSigned(self.REF)
        except:
            print "Error: Could not read reference pressure"
            return ""
            
    ## Reads the raw pressure value of the PPS58
    #  @param self The object pointer.
    def get_raw(self):
        try:
            return self.readLongSigned(self.RAW)
        except:
            print "Error: Could not read raw pressure"
            return ""
            
    ## Sets the reference pressure to the current absolute pressure value
    #  @param self The object pointer.
    def set_reference(self):
        try:
            self.command(68)
        except:
            print "Error: Could not set reference pressure"
            return ""
            
    ## Sets the unit of measure to pounds per square inch
    #  @param self The object pointer.
    def set_unit_PSI(self):
        try:
            self.command(80)
        except:
            print "Error: Could not set PSI as unit"
            return ""
            
    ## Sets the unit of measure to millibar
    #  @param self The object pointer.
    def set_unit_mbar(self):
        try:
            self.command(98)
        except:
            print "Error: Could not set millibar as unit"
            return ""
            
    ## Sets the unit of measure to kilopascal
    #  @param self The object pointer.
    def set_unit_kpascal(self):
        try:
            self.command(107)
        except:
            print "Error: Could not set kilopascal as unit"
            return ""  

## Volt: this class provides functions for NXTVoltMeter from mindsensors.com
#  for read and write operations.
class VOLT(mindsensors_i2c):  
    
    ## Default VoltMeter I2C Address
    VOLT_ADDRESS = (0x26)
    ## Command Register
    COMMAND = 0x41
    ## Absolute Calibrated Voltage value Register. Will Return a signed integer value
    CAL = 0x43
    ## Relative Voltage value Register. Will Return a signed integer value
    REL = 0x45
    ## Reference Voltage value Register. Will Return a signed integer value
    REF = 0x47
   
    ## Initialize the class with the i2c address of your NXTVoltMeter
    #  @param self The object pointer.
    #  @param i2c_address Address of your NXTVoltMeter.
    def __init__(self, volt_address = VOLT_ADDRESS):
        #the DIST address
        mindsensors_i2c.__init__(self, volt_address >> 1)
                
    ## Writes a value to the command register
    #  @param self The object pointer.
    #  @param commands Value to write to the command register.
    def command(self, cmd):
        self.writeByte(self.COMMAND, int(cmd)) 
    
    ## Reads the absolute voltage of the NXTVoltMeter
    #  @param self The object pointer.
    def get_calibrated(self):
        try:
            return self.readIntegerSigned(self.CAL)
        except:
            print "Error: Could not read calibrated voltage"
            return ""
        
    ## Reads the relative voltage of the NXTVoltMeter
    #  @param self The object pointer.
    def get_relative(self):
        try:
            return self.readIntegerSigned(self.REL)
        except:
            print "Error: Could not read relative voltage"
            return ""
        
    ## Reads the reference voltage of the NXTVoltMeter
    #  @param self The object pointer.
    def get_reference(self):
        try:
            return self.readIntegerSigned(self.REF)
        except:
            print "Error: Could not read reference voltage"
            return ""
            
    ## Sets the reference voltage to the current absolute voltage value
    #  @param self The object pointer.
    def set_reference(self):
        try:
            self.command(68)
        except:
            print "Error: Could not set reference voltage"
            return ""


import os,sys,inspect,time
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0,parentdir) 
from PiStorms import PiStorms

psm = PiStorms()

psm.screen.termPrintln("EV3 SumoEyes readout (BAS1):")
psm.screen.termPrintln(" ")
#print psm.BAS1.SE_None
exit = False
while(not exit):
    psm.screen.termPrintAt(4," SumoEyes " +str(psm.BAS1.SumoEyes(True)))
    '''
    if psm.BAS1.SumoEyes(True) == psm.BAS1.SumoEyes.SE_None :
        psm.screen.termPrintAt(4," SumoEyes None infront")
    if psm.BAS1.SumoEyes(True) == psm.BAS1.SE_Front :
        psm.screen.termPrintAt(4," SumoEyes something infront")
    if psm.BAS1.SumoEyes(True) == psm.BAS1.SE_Left :
        psm.screen.termPrintAt(4," SumoEyes something on Left")
    if psm.BAS1.SumoEyes(True) == psm.BAS1.SE_Right :
        psm.screen.termPrintAt(4," SumoEyes something on Right")
    '''
    if(psm.screen.checkButton(0,0,320,320)):
		psm.screen.termPrintln("")
		psm.screen.termPrintln("Exiting to menu")
		exit = True
        



#!/usr/bin/env python
#
# Copyright (c) 2015 mindsensors.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date      Author      Comments
# Jan 2016   Nitin     Initial Authoring 

from picamera.array import PiRGBArray
from picamera import PiCamera
import os,sys,inspect,time#thread
import cv2
import imutils
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0,parentdir) 
from PiStorms import PiStorms
psm = PiStorms()


print "running program"
psm.screen.termPrintln(" ")

# Get user supplied values
#cascPath = sys.argv[1]

# Create the haar cascade
#faceCascade = cv2.CascadeClassifier(cascPath)
faceCascade = cv2.CascadeClassifier("/home/pi/PiStorms/programs/haarcascade_frontalface_default.xml")

# initialize the camera and grab a reference to the raw camera capture
#down size the image resolution for imporving speed. 
try:
    camera = PiCamera()
except:
    m = ["PopHeads", "Camera not enabled.", "Run raspi-config and enable camera"]
    psm.screen.askQuestion(m,["OK"])
    exit()
    
camera.resolution = (320, 240)
camera.framerate = 32
rawCapture = PiRGBArray(camera, size=(320, 240))

# allow the camera to warmup
time.sleep(0.1)

#create milisecond time variable for benchmarking
lastTime = time.time()*1000.0

# capture frames from the camera
lastfaces = {}
for frame in camera.capture_continuous(rawCapture, format="bgr", use_video_port=True):

	# grab the raw NumPy array representing the image, then initialize the timestamp
	# and occupied/unoccupied text
    image = frame.array
    (imh, imw) = image.shape[:2]
    
    #convert the image from color to Grayscale space
    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
    
    # Detect faces in the image
    faces = faceCascade.detectMultiScale(
    gray,
    scaleFactor=1.1,
    minNeighbors=5,
    minSize=(30, 30),
    flags = cv2.cv.CV_HAAR_SCALE_IMAGE
    )
    
    #print benchmarking information on Console
    print time.time()*1000.0-lastTime," Found {0} faces!".format(len(faces))
    lastTime = time.time()*1000.0
    
    # Draw a rectangle around the faces
    
    for (x, y, w, h) in lastfaces:
        psm.screen.fillCircle(imw-(x+w), y+h, int((w+h)/3), fill = (0,0,0),display = True)
    
    for (x, y, w, h) in faces:
        # for displaying on exported display
        cv2.circle(image, (x+w/2, y+h/2), int((w+h)/3), (255, 255, 255), 1)
        
        # for showing on PiStorms screen
        psm.screen.fillCircle(imw-(x+w), y+h, int((w+h)/3), fill = (255,255,255),display = False)
        psm.screen.fillCircle(imw-(x+w), y+h, int(-2+(w+h)/3), fill = (0,0,0),display = False)
        psm.screen.fillCircle(imw-(x+w)+w/4, y+h-h/4, 4, fill = (255,255,255),display = False)
        psm.screen.fillCircle(imw-(x+w)-w/4, y+h-h/4, 4, fill = (255,255,255),display = True)
        
        
    # show the frame use this if you are setup for Display export on your pc or VNC
    #you may use Xming server for this
    
    #cv2.imshow("Frame", image)
    key = cv2.waitKey(1) & 0xFF
    lastfaces = faces
 
	# clear the stream in preparation for the next frame
    rawCapture.truncate(0)
    
	# if the `q` key was pressed, break from the loop
    if(psm.isKeyPressed() == True):
        break
    #if key == ord("q"):
    #    break
        
  
        




#!/usr/bin/env python
#
# Copyright (c) 2015 mindsensors.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date      Author      Comments
#  July 2015  Henry     Initial Authoring from PiStorms import PiStorms
# 10/18/15   Deepak     UI improvements and messenger integration

from PiStorms import PiStorms
from mindsensors_i2c import mindsensors_i2c
import sys, os, time, json

print str(sys.argv[1])
PROGRAM_DIRECTORY = str(sys.argv[1])
json_file = '/var/tmp/ps_data.json'
rotation = 3 
if(os.getenv("PSREVERSE","0")=="1"):
    rotation = 3
#print os.getcwd()

try:
    bootmode = mindsensors_i2c(0xEA>>1) 
    bootmode.readbyte()
    psm = PiStorms("PiStorms",rotation)
    psm.screen.termPrintAt(4,"PiStorms in fw upgrade mode")
    
except:
    psm = PiStorms("PiStorms",rotation) 

#sudo 
def listPrograms(directory):
    files =  os.listdir( directory)
    x = 0
    while(x<len(files)):
        if(not files[x].endswith(".py")) or (not files[x][0:2].isdigit()):
            del files[x]
           
        else:
            #if not(files[x][0:2].isdigit()):
            #    files[x] =  '0' + files[x][0:]
            files[x] = files[x][0:len(files[x])-3]
            x += 1

    return sorted(files)

def newMessageExists():
    try:
        f = open(json_file, 'r')
        try:
            data = json.loads(f.read())
            s = data['status']
            f.close()
        except:
            # no json in the file (or file missing)
            s = ""
    except:
            # no json in the file (or file missing)
            s = ""        
    if ( s == 'New' ):
        return True
    else:
        return False

def message_update_status( json_data, new_status ):
    f = open(json_file, 'w')
    json_data['status'] = new_status
    json.dump(json_data, f)
    f.close()

def runProgram(progName,progDir):
    psm.screen.clearScreen()
    return os.system("sudo python " +   progDir + "/" + progName + ".py")
    
    
def displaySmallFileList(fileList, displayLeft = True):
    initialYpos = 50
    xpos = 50
    height = 45
    width = 225
    
    psm.screen.clearScreen(display=False)
    psm.screen.drawDisplay("PiStorms",display=False)
    counter = 0
    while(counter<4 and counter<len(fileList)):
        psm.screen.drawButton(xpos,initialYpos + (height*counter),width=width,height=height,text=fileList[counter], display=False)
        counter += 1
    pageXPos = 0
    pageYPos = 0
    pageWidth = 50
    pageHeight = 50
    if(displayLeft):
        psm.screen.drawButton(pageXPos, pageYPos, width=pageWidth, height=pageHeight, text="<", display=False)

    psm.screen.drawButton((320-pageXPos)-pageWidth, pageYPos, pageWidth, pageHeight, text=">", display=False)

    newMessage = newMessageExists()
    if ( newMessage == True ):
        psm.screen.fillBmp(220,7,34,34, "Exclamation-mark-icon.png", False);
    
    # display the buffered data on screen.
    psm.screen.disp.display()

    while(True):
        
        if ( newMessage ):
            # handle the exclamation button
            if ( psm.screen.checkButton(218,5,38,38)):
                # exclamation button was clicked
                return "message"

        if(displayLeft and psm.screen.checkButton(pageXPos,pageYPos,pageWidth,pageHeight)):
            return 4
        if(psm.screen.checkButton((320-pageXPos)-pageWidth,pageYPos,pageWidth,pageHeight)):
            return 5
        counter = 0
        while(counter<4 and counter<len(fileList)):
            if(psm.screen.checkButton(xpos,initialYpos + (height*counter),width=width,height=height)):
                return counter
            counter += 1
            
def displayFullFileList(fileList,index = 0):
    if(index*4>len(fileList)):
        return displayFullFileList(fileList,0)
    if(index <0):
        return displayFullFileList(fileList,0)
    
    result = 0
    displayLeft = index != 0
    if(index*4+4<len(fileList)):
        result = displaySmallFileList(fileList[index*4:index*4+4],displayLeft)
    else:
        result = displaySmallFileList(fileList[index*4:len(fileList)],displayLeft)
    if(result == 4):
        return displayFullFileList(fileList,index - 1)
    if(result == 5):
        return displayFullFileList(fileList,index + 1)
    
    try:
        newResult = result + (index*4)
    except TypeError:
        newResult = result

    return newResult

#
# main program loop
#
try:
    while(True):
        result = 0
        #if(psm.battVoltage()<=6.5):
        #    psm.screen.askQuestion(["LOW BATTERY","Your battery is low","Change or charge your batteries"],["Ignore"])
        #if(psm.isKeyPressed()):
        #     psm.screen.refresh()
        files = listPrograms(PROGRAM_DIRECTORY)
        file_id = displayFullFileList(files)
        if ( isinstance( file_id, int ) ):
            result = runProgram(files[file_id],PROGRAM_DIRECTORY)
        elif ( file_id == "message"):
            f = open(json_file, 'r')
            try:
                data = json.loads(f.read())
                m = data['message'].split("\n")
                s = data['status']
                f.close()
            except:
                pass
            message_update_status( data, "Read" )
            psm.screen.askQuestion(m,["OK"])
            result = 0 

        psm.BAM1.float()
        psm.BAM2.float()
        psm.BBM1.float()
        psm.BBM2.float()
        if(result != 0):
            psm.screen.refresh()
            psm.screen.askQuestion(["ERROR","Program exited with error.","(Error Code " + str(result) + ")"],["OK"])
            
except KeyboardInterrupt:
    print "Quitting..."
    psm.screen.refresh()
    psm.screen.termPrintln("PiStormsBrowser Exited")



#!/usr/bin/env python
#
# Copyright (c) 2015 mindsensors.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date        Author      Comments
#  04-11-16   Deepak      Troubleshooting program

import time

from PiStorms import PiStorms
print ""
print "Running program 5 times, press Ctrl-C to terminate"
print ""
psm = PiStorms()

exit = False
a = 0
while(a < 5):
    voltVal = psm.battVoltage()
    print "PiStorms voltage: " + str(voltVal)
    time.sleep(2)
    a = a + 1



#!/usr/bin/env python
#
# Copyright (c) 2015 mindsensors.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date      Author      Comments
# Oct 2015  Michael     Initial Authoring

import os,sys,inspect,time,thread
import socket,fcntl,struct    

currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0,parentdir) 
from PiStorms import PiStorms

print "running program"
psm = PiStorms()
psm.screen.drawButton(75, 70, width = 75, height = 40, text="Greet", display=False)
psm.screen.drawButton(75, 115, width = 75, height = 40, text="Exit", display=True)

exit = False
lastled = 0

while(not exit):
    greet = psm.screen.checkButton(75, 70,width=75,height=40)
    bye = psm.screen.checkButton(75, 115,width=75,height=40)
    if(greet == True):
        psm.screen.drawAutoText("Hello World!", 50, 175,fill = (255,255,255), size = 25, display = True, align = "center")
    if(bye == True):
        psm.screen.termPrintln("")
        psm.screen.drawAutoText("Exiting to menu", 50, 210,fill = (255,255,255), size = 20, display = True, align = "left")
        exit = True


import os,sys,inspect,time

currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0,parentdir) 

from PiStorms import PiStorms

psm = PiStorms()

psm.screen.termPrintln("EV3 touch sensor readout (BBS1):")
psm.screen.termPrintln(" ")

psm.BBS1.resetTouchesEV3()
exit = False

while(not exit):
    touch = psm.BBS1.isTouchedEV3()
    numTouch = psm.BBS1.numTouchesEV3()
	psm.screen.termReplaceLastLine(str(touch) + "  " + str(numTouch))
	if(psm.screen.checkButton(0,0,320,320)):
		psm.screen.termPrintln("")
		psm.screen.termPrintln("Exiting to menu")
		exit = True



#!/usr/bin/env python
#
# Copyright (c) 2015 mindsensors.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date      Author      Comments
#  Aug 2015  Andrew     Initial Authoring from SensorShield import SensorShield


import os,sys,inspect,time,thread,random
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0,parentdir) 
from PiStorms import PiStorms

# starts an instance of PiStorms
psm = PiStorms()

### setup variables ###
# exit is used to eventually exit the main running loop
exit = False

# sets screen to a white background
psm.screen.fillRect(0, 0, 320, 240)

# initial loading screen
psm.screen.fillBmp(50, 20, 100, 100, path = currentdir+'/'+"faceClown.png")
psm.screen.drawAutoText("Hi There!", 15, 140, fill=(0, 0, 0), size = 30)
psm.screen.drawAutoText("poke my eyes or tap my nose", 15, 170, fill=(0, 0, 0), size = 20)
time.sleep(.5)
psm.screen.drawAutoText("3", 50,200,fill=(0,0,0),size=25)
time.sleep(.5)
psm.screen.fillRect(50,200,25,25)
psm.screen.drawAutoText("2", 50,200,fill=(0,0,0),size=25)
time.sleep(.5)
psm.screen.fillRect(50,200,25,25)
psm.screen.drawAutoText("1", 50,200,fill=(0,0,0),size=25)
time.sleep(.5)
psm.screen.fillRect(50,200,25,25)
psm.screen.drawAutoText("GO", 50,200,fill=(0,0,0),size=25)
time.sleep(.5)

currFace = -1

# sets screen to a white background
psm.screen.fillRect(0, 0, 320, 240)

while(not exit):
    #plain face
    if(currFace != 0):
        psm.screen.fillBmp(40, 0, 240, 240, path = currentdir+'/'+"faceClown.png")
        currFace = 0
        
    #right eye
    if(psm.screen.checkButton(106, 68, 44, 53)):
        psm.screen.fillBmp(40,0,240,240, path = currentdir+'/'+"faceClown_eyeRight.png")
        currFace = 1
        
    #left eye
    if(psm.screen.checkButton(175, 68, 44, 53)):
        psm.screen.fillBmp(40,0,240,240, path = currentdir+'/'+"faceClown_eyeLeft.png")
        currFace = 1
    
    #nose
    if(psm.screen.checkButton(130, 122, 70, 60)):
        psm.screen.fillBmp(40,0,240,240, path = currentdir+'/'+"faceClown_nose.png")
        currFace = 1
    
    if(psm.isKeyPressed() == True): # if the GO button is pressed
        psm.screen.clearScreen()
        psm.screen.termPrintln("") 
        psm.screen.termPrintln("Exiting to menu")
        time.sleep(0.5) 
        exit = True 


#!/usr/bin/env python
#
# Copyright (c) 2015 mindsensors.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/
# History:
# Date      Author      Comments
# 09/11/15  Nitin Patil     Initial Authoring
#
#Demo Code for the PiStorms and Raspberry Pi


#initial setup code
import os,sys,inspect,time,thread, random
from mindsensors_i2c import mindsensors_i2c
from PiStorms import PiStorms
from mindsensors import ABSIMU

currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0,parentdir) 

#starts an instance of PiStorms

imu=ABSIMU()
psm = PiStorms()
#exit variable will be used later to exit the program and return to PiStormsMaster
exit = False

#clears the screen of any unwanted text by displaying a white rectangle
#psm.screen.fillRect(0, 0, 320, 240)
psm.screen.termPrintln("Custom I2C test program")
psm.screen.termPrintln("connect mindsensors.com's ")
psm.screen.termPrintln("AbsImu on port BBS1 ")
psm.led(1, 0,0,0)
psm.led(2, 0,0,0)

psm.BBS1.activateCustomSensorI2C() #Connect the I2C sensor on the port BBS1
#main loop
# This test program will print IMU data on Terminal 
# Compass heading is represented on Red LED on BANKB 
# Program will exit when someone touch the screen or Go Button

while(not exit):
    #
    try:
        heading = imu.get_heading()
        accl = imu.get_accelall()
        mag = imu.get_magall()
        gyro = imu.get_gyroall()
        psm.led(2,heading/3, 0,0)
        psm.screen.termPrintAt(6," Heading to "+ str(heading) +" degree")
    except:
        psm.led(2,0, 0,0)
        psm.screen.termPrintAt(6,"connect AbsImu on port BBS1 ")
     
    print " Accelerometer: " + str(accl)
    print " Magnetometer" + str(mag)
    print " Gyroscope: " + str(gyro)
    print " Compass: " + str(heading)
    
    time.sleep(.1)    
       
        
    if(psm.isKeyPressed() == True): # if the GO button is pressed
        psm.screen.clearScreen()
        psm.screen.termPrintln("") 
        psm.screen.termPrintln("Exiting to menu")
        time.sleep(0.5) 
        psm.led(1, 0,0,0)
        psm.led(2, 0,0,0)
        exit = True 


#!/usr/bin/env python
#
# Copyright (c) 2014 OpenElectrons.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/
#

# History:
# Date       Author           Comments
# 07/26/14   Michael Giles   Initial authoring.
# 08/18/14   Michael Giles   Servo 
# 10/18/15   Nitin           PiStorm integration
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import PiStorms
import time,sys 

dict = {}
read_dict = {}
check_dict = {}   
                           
## rmap_PISTORMS: this class provides functions for PISTORMS integration with scratch
#  for read and write operations with scratch.
class rmap_PISTORMS(PiStorms.PiStorms):
    
    def __init__(self):
        PiStorms.PiStorms.__init__(self,"PiStorms") 
        self.screen.termPrintAt(0,"     Scratch Interface Program")        
       
    def process_read(self, cmd_string):
        if ( 'EV3LIGHT' in cmd_string ):                
            try:
                if ( 'BAS1' in cmd_string ): 
                    r = self.BAS1.reflectedLightSensorEV3()
                elif ( 'BAS2' in cmd_string ): 
                    r = self.BAS2.reflectedLightSensorEV3()
                elif ( 'BBS1' in cmd_string ): 
                    r = self.BBS1.reflectedLightSensorEV3()
                elif ( 'BBS2' in cmd_string ): 
                    r = self.BBS2.reflectedLightSensorEV3()
                #print cmd_string ,r
            #    check_dict[cmd_string] = r
                return r
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return "" 
        elif ( 'EV3AMBIENTLIGHT' in cmd_string ):                
            try:
                if ( 'BAS1' in cmd_string ): 
                    r = self.BAS1.ambientLightSensorEV3()
                elif ( 'BAS2' in cmd_string ): 
                    r = self.BAS2.ambientLightSensorEV3()
                elif ( 'BBS1' in cmd_string ): 
                    r = self.BBS1.ambientLightSensorEV3()
                elif ( 'BBS2' in cmd_string ): 
                    r = self.BBS2.ambientLightSensorEV3()
                #print cmd_string ,r
                check_dict[cmd_string] = r
                return r   
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""    
        elif ( 'EV3COLOR' in cmd_string ):   
            #print cmd_string
            try:
                if ( 'BAS1' in cmd_string ): 
                    r = self.BAS1.colorSensorEV3()
                elif ( 'BAS2' in cmd_string ): 
                    r = self.BAS2.colorSensorEV3()
                elif ( 'BBS1' in cmd_string ): 
                    r = self.BBS1.colorSensorEV3()
                elif ( 'BBS2' in cmd_string ): 
                    r = self.BBS2.colorSensorEV3()
                #print cmd_string ,r
            #    check_dict[cmd_string] = r
                return r
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""                 
        elif ( 'EV3TOUCHED' in cmd_string ):                
            try:
                if ( 'BAS1' in cmd_string ): 
                    r = self.BAS1.isTouchedEV3()
                elif ( 'BAS2' in cmd_string ): 
                    r = self.BAS2.isTouchedEV3()
                elif ( 'BBS1' in cmd_string ): 
                    r = self.BBS1.isTouchedEV3()
                elif ( 'BBS2' in cmd_string ): 
                    r = self.BBS2.isTouchedEV3()
                #print cmd_string ,r
                check_dict[cmd_string] = r
                return r   
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""                 
        elif ( 'EV3TOUCHES' in cmd_string ):                
            try:
                if ( 'BAS1' in cmd_string ): 
                    r = self.BAS1.numTouchesEV3()
                elif ( 'BAS2' in cmd_string ): 
                    r = self.BAS2.numTouchesEV3()
                elif ( 'BBS1' in cmd_string ): 
                    r = self.BBS1.numTouchesEV3()
                elif ( 'BBS2' in cmd_string ): 
                    r = self.BBS2.numTouchesEV3()
                #print cmd_string ,r
                check_dict[cmd_string] = r
                return r  
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""                 
        elif ( 'EV3IRDISTANCE' in cmd_string ):       
            try:
                if ( 'BAS1' in cmd_string ): 
                    r = self.BAS1.distanceIREV3()
                elif ( 'BAS2' in cmd_string ): 
                    r = self.BAS2.distanceIREV3()
                elif ( 'BBS1' in cmd_string ): 
                    r = self.BBS1.distanceIREV3()
                elif ( 'BBS2' in cmd_string ): 
                    r = self.BBS2.distanceIREV3()
                #print cmd_string ,r
                check_dict[cmd_string] = r
                return r  
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""                 
        elif ( 'EV3IRHEADING' in cmd_string ):     
            try:
                if ( 'BAS1' in cmd_string ):          
                    r = self.BAS1.headingIREV3(int(cmd_string[3]))
                elif ( 'BAS2' in cmd_string ): 
                    r = self.BAS2.headingIREV3(int(cmd_string[3]))
                elif ( 'BBS1' in cmd_string ): 
                    r = self.BBS1.headingIREV3(int(cmd_string[3]))
                elif ( 'BBS2' in cmd_string ): 
                    r = self.BBS2.headingIREV3(int(cmd_string[3]))
                #print cmd_string ,r
                check_dict[cmd_string] = r
                return r 
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""                 
        elif ( 'REMOTELEFT' in cmd_string ):                
            try:
                if ( 'BAS1' in cmd_string ): 
                    r = self.BAS1.remoteLeft(int(cmd_string[3]))
                elif ( 'BAS2' in cmd_string ): 
                    r = self.BAS2.remoteLeft(int(cmd_string[3]))
                elif ( 'BBS1' in cmd_string ): 
                    r = self.BBS1.remoteLeft(int(cmd_string[3]))
                elif ( 'BBS2' in cmd_string ): 
                    r = self.BBS2.remoteLeft(int(cmd_string[3]))
                #print cmd_string ,r
                check_dict[cmd_string] = r
                return r   
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""     
        elif ( 'REMOTERIGHT' in cmd_string ):                
            try:
                if ( 'BAS1' in cmd_string ): 
                    r = self.BAS1.remoteRight(int(cmd_string[3]))
                elif ( 'BAS2' in cmd_string ): 
                    r = self.BAS2.remoteRight(int(cmd_string[3]))
                elif ( 'BBS1' in cmd_string ): 
                    r = self.BBS1.remoteRight(int(cmd_string[3]))
                elif ( 'BBS2' in cmd_string ): 
                    r = self.BBS2.remoteRight(int(cmd_string[3]))
                #print cmd_string ,r
                check_dict[cmd_string] = r
                return r   
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""  
        elif ( 'EV3ULTRASONIC' in cmd_string ):                
            try:
                if ( 'BAS1' in cmd_string ): 
                    r = self.BAS1.distanceUSEV3()
                elif ( 'BAS2' in cmd_string ): 
                    r = self.BAS2.distanceUSEV3()
                elif ( 'BBS1' in cmd_string ): 
                    r = self.BBS1.distanceUSEV3()
                elif ( 'BBS2' in cmd_string ): 
                    r = self.BBS2.distanceUSEV3()
                #print cmd_string ,r
                check_dict[cmd_string] = r
                return r   
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""    
        elif ( 'EV3GYROANGLE' in cmd_string ):                
            try:
                if ( 'BAS1' in cmd_string ): 
                    r = self.BAS1.gyroAngleEV3()
                elif ( 'BAS2' in cmd_string ): 
                    r = self.BAS2.gyroAngleEV3()
                elif ( 'BBS1' in cmd_string ): 
                    r = self.BBS1.gyroAngleEV3()
                elif ( 'BBS2' in cmd_string ): 
                    r = self.BBS2.gyroAngleEV3()
                #print cmd_string ,r
                check_dict[cmd_string] = r
                return r   
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""    
        elif ( 'EV3GYRORATE' in cmd_string ):                
            try:
                if ( 'BAS1' in cmd_string ): 
                    r = self.BAS1.gyroRateEV3()
                elif ( 'BAS2' in cmd_string ): 
                    r = self.BAS2.gyroRateEV3()
                elif ( 'BBS1' in cmd_string ): 
                    r = self.BBS1.gyroRateEV3()
                elif ( 'BBS2' in cmd_string ): 
                    r = self.BBS2.gyroRateEV3()
                #print cmd_string ,r
                check_dict[cmd_string] = r
                return r   
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""         
        elif ( 'NXTLIGHT' in cmd_string ):                
            try:
                if ( 'BAS1' in cmd_string ): 
                    r = self.BAS1.lightSensorNXT(True)
                elif ( 'BAS2' in cmd_string ): 
                    r = self.BAS2.lightSensorNXT(True)
                elif ( 'BBS1' in cmd_string ): 
                    r = self.BBS1.lightSensorNXT(True)
                elif ( 'BBS2' in cmd_string ): 
                    r = self.BBS2.lightSensorNXT(True)
                #print cmd_string ,r
                check_dict[cmd_string] = r
                return r   
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return "" 
        elif ( 'NXTAMBIENTLIGHT' in cmd_string ):                
            try:
                if ( 'BAS1' in cmd_string ): 
                    r = self.BAS1.lightSensorNXT(False)
                elif ( 'BAS2' in cmd_string ): 
                    r = self.BAS2.lightSensorNXT(False)
                elif ( 'BBS1' in cmd_string ): 
                    r = self.BBS1.lightSensorNXT(False)
                elif ( 'BBS2' in cmd_string ): 
                    r = self.BBS2.lightSensorNXT(False)
                #print cmd_string ,r
                check_dict[cmd_string] = r
                return r   
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""                 
        elif ( 'NXTCOLOR' in cmd_string ):                
            try:
                if ( 'BAS1' in cmd_string ): 
                    r = self.BAS1.colorSensorNXT()
                elif ( 'BAS2' in cmd_string ): 
                    r = self.BAS2.colorSensorNXT()
                elif ( 'BBS1' in cmd_string ): 
                    r = self.BBS1.colorSensorNXT()
                elif ( 'BBS2' in cmd_string ): 
                    r = self.BBS2.colorSensorNXT()
                #print cmd_string ,r
                check_dict[cmd_string] = r
                return r   
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""  
        elif ( 'NXTTOUCHED' in cmd_string ): 
            #print         cmd_string
            try:
                if ( 'BAS1' in cmd_string ): 
                    r = self.BAS1.isTouchedNXT()
                elif ( 'BAS2' in cmd_string ): 
                    r = self.BAS2.isTouchedNXT()
                elif ( 'BBS1' in cmd_string ): 
                    r = self.BBS1.isTouchedNXT()
                elif ( 'BBS2' in cmd_string ): 
                    r = self.BBS2.isTouchedNXT()
                #print cmd_string ,r
                check_dict[cmd_string] = r
                return r   
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""                 
        elif ( 'NXTTOUCHES' in cmd_string ):   
            #print         cmd_string
            try:
                if ( 'BAS1' in cmd_string ): 
                    r = self.BAS1.numTouchesNXT()
                elif ( 'BAS2' in cmd_string ): 
                    r = self.BAS2.numTouchesNXT()
                elif ( 'BBS1' in cmd_string ): 
                    r = self.BBS1.numTouchesNXT()
                elif ( 'BBS2' in cmd_string ): 
                    r = self.BBS2.numTouchesNXT()
                #print cmd_string ,r
                check_dict[cmd_string] = r
                return r  
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""  

        elif ( 'SUMOEYES' in cmd_string ):                
            try:
                if ('SHORT' in cmd_string ):
                    range = False
                else:
                    range = True
                if ( 'BAS1' in cmd_string ): 
                    r = self.BAS1.SumoEyes(range)
                elif ( 'BAS2' in cmd_string ): 
                    r = self.BAS2.SumoEyes(range)
                elif ( 'BBS1' in cmd_string ): 
                    r = self.BBS1.SumoEyes(range)
                elif ( 'BBS2' in cmd_string ): 
                    r = self.BBS2.SumoEyes(range)
                #print cmd_string ,r
                check_dict[cmd_string] = r
                return r   
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""                    
        elif ( 'ANALOG' in cmd_string ):
            #print cmd_string     
            try:
                if ( 'BAS1' in cmd_string ): 
                    r = self.BAS1.analogSensor()
                elif ( 'BAS2' in cmd_string ): 
                    r = self.BAS2.analogSensor()
                elif ( 'BBS1' in cmd_string ): 
                    r = self.BBS1.analogSensor()
                elif ( 'BBS2' in cmd_string ): 
                    r = self.BBS2.analogSensor()
                #print cmd_string ,r
                check_dict[cmd_string] = r
                return r   
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""       
        elif ( 'ENCODER' in cmd_string ):                
            #print cmd_string
            try:
                if ( 'BAM1' in cmd_string ): 
                    r = self.BAM1.pos()
                elif ( 'BAM2' in cmd_string ): 
                    r = self.BAM2.pos()
                elif ( 'BAM3' in cmd_string ): 
                    r = self.BBM2.pos()
                elif ( 'BAM4' in cmd_string ): 
                    r = self.BBM2.pos()
                #print cmd_string ,r
                check_dict[cmd_string] = r
                return r   
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""                  
        elif ( 'BATTVOLT' in cmd_string ):                
            try:
                r = self.battVoltage()
                #print r
                check_dict[cmd_string] = r
                return r   
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""   
        elif ( 'GOBUTTON' in cmd_string ):
            if 'COUNT' in cmd_string :
                try:
                    r = self.getKeyPressCount() 
                    check_dict[cmd_string] = r
                    return r   
                except:
                    rmap_print("Could not read PiStorms")
                    #rmap_print("Check I2C address and device connection to resolve issue")
                return ""
            else:    
                try:
                    r = self.isKeyPressed() 
                    check_dict[cmd_string] = r
                    return r   
                except:
                    rmap_print("Could not read PiStorms")
                    #rmap_print("Check I2C address and device connection to resolve issue")
                    return "" 
        elif ( 'TOUCHX' in cmd_string ):
            #print cmd_string        
            try:
                r = self.screen.TS_X() #isKeyPressed()
                check_dict[cmd_string] = r
                return r   
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""
        elif ( 'TOUCHY' in cmd_string ):
            #print cmd_string        
            try:
                r = self.screen.TS_Y() #isKeyPressed()
                check_dict[cmd_string] = r
                return r   
            except:
                rmap_print("Could not read PiStorms")
                #rmap_print("Check I2C address and device connection to resolve issue")
                return ""                 

    def run_command(self, cmd_string, message):
        #print cmd_string, message
        if ( 'ON' in cmd_string ): 
            if len(cmd_string) > 3:
                speed = int(cmd_string[3])
                if speed > 100:
                    speed = 100
                if speed < -100:
                    speed = -100   
            else:
                speed = 100        
            if ( 'BAM1' in cmd_string ): 
                #print "self.BAM1.setSpeed(speed)", speed
                self.BAM1.setSpeed(speed)
            if ( 'BAM2' in cmd_string ): 
                self.BAM2.setSpeed(speed)
            if ( 'BBM1' in cmd_string ): 
                self.BBM1.setSpeed(speed)
            if ( 'BBM4' in cmd_string ): 
                self.BBM2.setSpeed(speed)
                
        elif ( 'OFF' in cmd_string ): 
            speed = 0       
            if ( 'BAM1' in cmd_string ): 
                self.BAM1.setSpeed(speed)
            if ( 'BAM2' in cmd_string ): 
                self.BAM2.setSpeed(speed)
            if ( 'BBM1' in cmd_string ): 
                self.BBM1.setSpeed(speed)
            if ( 'BBM2' in cmd_string ): 
                self.BBM2.setSpeed(speed)  
                
        elif ( 'BRAKE' in cmd_string ): 
            if ( 'BAM1' in cmd_string ): 
                self.BAM1.brake()
            if ( 'BAM2' in cmd_string ): 
                self.BAM2.brake()
            if ( 'BBM1' in cmd_string ): 
                self.BBM1.brake()
            if ( 'BBM2' in cmd_string ): 
                self.BBM2.brake()  
                
        elif ( 'FLOAT' in cmd_string ): 
            if ( 'BAM1' in cmd_string ): 
                self.BAM1.float()
            if ( 'BAM2' in cmd_string ): 
                self.BAM2.float()
            if ( 'BBM1' in cmd_string ): 
                self.BBM1.float()
            if ( 'BBM2' in cmd_string ): 
                self.BBM2.float()                 
        elif ( 'RUNSEC' in cmd_string ): 
            if len(cmd_string) > 4:
                speed = int(cmd_string[4])
                if speed > 100:
                    speed = 100
                if speed < -100:
                    speed = -100
                runtime = int(cmd_string[3])
                if runtime > 100:
                    runtime = 100
            else:
                runtime = 1     
            if ( 'BAM1' in cmd_string ): 
                self.BAM1.runSecs(runtime,speed,True)
            if ( 'BAM2' in cmd_string ): 
                self.BAM2.runSecs(runtime,speed,True)
            if ( 'BBM1' in cmd_string ): 
                self.BBM1.runSecs(runtime,speed,True)
            if ( 'BBM2' in cmd_string ): 
                self.BBM2.runSecs(runtime,speed,True)
        elif ( 'RUNDEG' in cmd_string ): 
            #print cmd_string
            if len(cmd_string) > 4:
                speed = int(cmd_string[4])
                if speed > 100:
                    speed = 100
                if speed < -100:
                    speed = -100
                deg = int(cmd_string[3])
                if deg > 10000:
                    deg = 10000
                if deg < -10000:
                    deg = -10000     
               
            else:
                deg = 360
                speed = 100        
            #print  speed,deg       
            if ( 'BAM1' in cmd_string ): 
                self.BAM1.runDegs(deg,speed,True,False)
            if ( 'BAM2' in cmd_string ): 
                self.BAM2.runDegs(deg,speed,True,False)
            if ( 'BBM1' in cmd_string ): 
                self.BBM1.runDegs(deg,speed,True,False)
            if ( 'BBM2' in cmd_string ): 
                self.BBM2.runDegs(deg,speed,True,False)
        elif ( 'PRINT' in cmd_string ): 
            #print cmd_string
            if len(cmd_string) > 2:
                line = int(cmd_string[2])
                #print message[2:-2]
                self.screen.termPrintAt(line,message)
        elif ( 'EXIT' in cmd_string ): 
            #print cmd_string
            quit(0)   
        else :
            return False
        return True    
           
                 
def device_cr(class_name, variable_name, address = 0):
    if (class_name == "PILIGHT") :
        return rmap_PILIGHT(0x30) #int(address, base=16)
    if (class_name == "SERVO") :
        if (address == '0'):
            return rmap_SERVO()   
    if (class_name == "PISTORMS") :
        return rmap_PISTORMS() #int(address, base=16)
        
def save_read_data(command_string):
    x = 0
    max = len(command_string) + 1 
    read_dict[command_string[1]] = []
    for cmd in command_string:
        if x > 1:
            read_dict[command_string[1]].append(command_string[x])
        if x < max:
            x = x + 1
            
def rmap_print(message):
    print str(time.strftime("%d/%m/%Y-"))+str(time.strftime("%H:%M:%S"))+":"+message
    sys.stdout.flush()

def print_dict():
    print dict
    



#!/usr/bin/env python
#
# Copyright (c) 2015 mindsensors.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date      Author      Comments
#  July 2015  Henry     Initial Authoring from SensorShield import SensorShield

import os,sys,inspect,time,thread

currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0,parentdir) 
from PiStorms import PiStorms
print "running program"
psm = PiStorms()

psm.screen.termPrintln("Battery Voltage")
psm.screen.termPrintln(" ")

psm.BBS1.resetTouchesEV3()
exit = False
lastled = 0
while(not exit):
    voltVal = psm.battVoltage()
    psm.screen.termReplaceLastLine(str(voltVal) + "V")
	
    if(voltVal >= 8and lastled != 1):
        psm.led(1,0,255,0)
        psm.led(2,0,255,0)
        lastled = 1
    if(voltVal < 8 and voltVal > 6 and lastled != 2):
        psm.led(1,0,255,0)
        psm.led(2,0,255,0)
        lastled = 2
    if(voltVal <= 6 and lastled != 3):
        psm.led(1,255,0,0)
        psm.led(2,255,0,0)
        lastled = 3
    if(psm.screen.checkButton(0,0,320,320)):
        psm.screen.termPrintln("")
        psm.screen.termPrintln("Exiting to menu")
        psm.led(1,0,0,0)
        psm.led(2,0,0,0)
        exit = True
		


#!/usr/bin/env python
#
# Copyright (c) 2015 mindsensors.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date      Author      Comments
# September 20, 2015  Nitin Patil     Initial Authoring 

from PiStorms import PiStorms

#Demo Code for the PiStorms and Raspberry Pi


#initial setup code
import os,sys,inspect,time,thread, random
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
#sys.path.insert(0,parentdir) 
from PiStorms  import PiStorms 

#starts an instance of PiStorms 
psm = PiStorms (rotation =1)

#exit variable will be used later to exit the program and return to PiStorms 
exit = False

#clears the screen of any unwanted text by displaying a white rectangle
psm.screen.fillRect(0, 0, 320, 240)

#displays PiDoG's greeting of "Hello, I PiDoG"
psm.screen.drawAutoText("Hello", 80, 30, fill=(255, 0, 0), size = 70)
psm.screen.drawAutoText("I am PyDog", 50, 140, fill=(255, 0, 0), size = 45)
count = 0


#main loop
while(not exit):
    
    if(psm.BBS2.distanceUSEV3() < 200): #if ultrasonic sensor reading is <200 (~6")
        
        #play Barking sound
        psm.screen.fillBmp(30, 0, 240, 240, path = currentdir+'/'+"dog.png")
        os.system('mpg123 -q  -f55000 /home/pi/PiStormsprograms/Puppy_Dog_Barking.mp3 &')
        mytime = int(round(time.time())) 
        
        while (int(round(time.time()))  - mytime ) < 10:
            #Quickly wag dog tail
            count = 0
            while count < 5:
                #psm.BAM2.setSpeed(-5);
                psm.BBM1.setSpeed(10);
                psm.BAM1.setSpeed(-10);
                time.sleep(0.05)
                #psm.BAM2.setSpeed(5);
                psm.BBM1.setSpeed(-10);
                psm.BAM1.setSpeed(10);
                time.sleep(0.05)
                count = count + 1
            psm.BAM2.setSpeed(0); 
            psm.BBM1.setSpeed(0); 
            psm.BAM1.setSpeed(0); 
            count = 0
            while count < 10:
                psm.BAM2.setSpeed(-5);
                #psm.BBM1.setSpeed(15);
                #psm.BAM1.setSpeed(-15);
                time.sleep(0.2)
                psm.BAM2.setSpeed(5);
                #psm.BBM1.setSpeed(-15);
                #psm.BAM1.setSpeed(15);
                time.sleep(0.2)
                count = count + 1
            psm.BAM2.setSpeed(0); 
            psm.BBM1.setSpeed(0); 
            psm.BAM1.setSpeed(0);         
            time.sleep(1)
 
        
        #clear screen of scared emoticon
        psm.screen.fillRect(0, 0, 320, 240)
        
              
        
        #redraw "Hello I am Sam"        
        psm.screen.drawAutoText("Hello", 80, 30, fill=(255, 0, 0), size = 70)
        psm.screen.drawAutoText("I am PyDog", 50, 140, fill=(255, 0, 0), size = 45)
        
        
    if(psm.isKeyPressed() == True): # if the GO button is pressed
        psm.screen.clearScreen()
        psm.screen.termPrintln("") 
        psm.screen.termPrintln("Exiting to menu")
        time.sleep(0.5) 
        exit = True 


#!/usr/bin/env python

from distutils.core import setup

setup(name='PiStorms',
    version='3.05',
    description='PiStorms libraries',
    author='mindsensors.com',
    author_email='contact@mindsensors.com',
    url='http://www.mindsensors.com',
    py_modules=['PiStorms', 'mindsensors', 'PiStormsCom', 'ps_messenger_check', 'scratch', 'rmap', 'rmapcfg', 'PiStormsInput'],
    data_files=[('mindsensors_images', ['btns_center.png', 'btns_left.png', 'btns_right.png', 'button.png', 'Exclamation-mark-icon.png', 'Pane1.png']),
    ('/etc/init.d', ['PiStormsDriver.sh', 'PiStormsBrowser.sh']),
    ('/home/pi/PiStorms/programs', ['00-About_Me.py', '00-Scratch_PiStorms.py', '00-RevertWiFiChanges.py', '00-TestInternetConn.py', '00-WiFi_Setup.py', '01-PopHeads.py', '01-TestKeyboard.py', '01-JesterControl.py', '02-SamTheEmotional.py', '01-CatchMike.py', '02-PyDog.py', '05-custom_i2c_test.py', '05-BatteryVolt.py', '06-SumoEyes.py', '06-touch_sensor.py', '04-GoButton.py', '04-HelloWorld.py', '09-Change_i2c_addr.py', '09-Explorer.py', '09-refresh.py', 'dog.jpg', 'dog.png', 'faceAwesome.png', 'faceClown.png', 'faceClown_eyeLeft.png', 'faceClown_eyeRight.png', 'faceClown_nose.png', 'faceHappy.png', 'faceScared2.png', 'Puppy_Dog_Barking.mp3', 'addresschange', '03-Swarm_Demo.py', 'smiley.png', 'black-square.png', 'lock.png', 'ulock.png', 'load.png', 'x_red.png', 'wifi_green.png', 'dialogbg.png', 'haarcascade_frontalface_default.xml']),
	('/home/pi/PiStorms/programs/tests', ['msg-to-screen.py', 'print-battery-voltage.py', 'psm-info.py', 'screen-test.py']),
	('/home/pi/Documents/Scratch Projects/PiStorms',['PiStorms-EV3AmbientLight.sb', 'PiStorms-EV3Color.sb', 'PiStorms-EV3Gyro.sb', 'PiStorms-EV3IRDistance.sb', 'PiStorms-EV3IRRemote.sb', 'PiStorms-EV3TouchSensor.sb', 'PiStorms-EV3Ultrasonic.sb', 'PiStorms-JoyStick.sb', 'PiStorms-MotorDemo.sb', 'PiStorms-NXTAmbientLight.sb', 'PiStorms-NXTColor.sb', 'PiStorms-NXTTouchSensor.sb', 'PiStorms-printing.sb', 'PiStorms-ReadEncoder.sb', 'PiStorms-RemoteRobot.sb', 'PiStorms-Sumoeyes.sb', 'PiStorms-Template.sb', 'PiStorms-TouchScreen.sb', 'PiStorms-TouchSensor.sb']),
    ('/usr/local/bin', ['pistorms-diag.sh', 'PiStormsBrowser.py', 'PiStormsDriver.py']),
	('/usr/share/raspberrypi-artwork', ['mindsensors-logo-BG.png', 'mindsensors-logo-Menu.png']),
    ('/home/pi/.config/autostart', ['tightvnc.desktop'])],
    install_requires=['mindsensors_i2c', 'mindsensorsUI', 'RPi.GPIO', 'imutils', 'wireless', 'wifi', 'cv2', 'swarmclient'],
    )


import array
import itertools
import socket
import struct

class ScratchError(Exception): pass
class ScratchConnectionError(ScratchError): pass	

class Scratch(object):

    prefix_len = 4
    broadcast_prefix_len = prefix_len + len('broadcast ')
    sensorupdate_prefix_len = prefix_len + len('sensor-update ')

    msg_types = set(['broadcast', 'sensor-update'])

    def __init__(self, host='localhost', port=42001):
	self.host = host
	self.port = port
	self.socket = None
	self.connected = False
	self.connect()

    def __repr__(self):
	return "Scratch(host=%r, port=%r)" % (self.host, self.port)

    def _pack(self, msg):
	"""
	Packages msg according to Scratch message specification (encodes and 
	appends length prefix to msg). Credit to chalkmarrow from the 
	scratch.mit.edu forums for the prefix encoding code.
	"""
	n = len(msg) 
	a = array.array('c')
	a.append(chr((n >> 24) & 0xFF))
	a.append(chr((n >> 16) & 0xFF))
	a.append(chr((n >>  8) & 0xFF))
	a.append(chr(n & 0xFF))
	return a.tostring() + msg 

    def _extract_len(self, prefix):
	"""
	Extracts the length of a Scratch message from the given message prefix. 
	"""
	return struct.unpack(">L", prefix)[0]

    def _get_type(self, s):
	"""
	Converts a string from Scratch to its proper type in Python. Expects a
	string with its delimiting quotes in place. Returns either a string, 
	int or float. 
	"""
	# TODO: what if the number is bigger than an int or float?
	if s.startswith('"') and s.endswith('"'):
	    return s[1:-1]
	elif s.find('.') != -1: 
	    return float(s) 
	else: 
	    return int(s)

    def _escape(self, msg):
	"""
	Escapes double quotes by adding another double quote as per the Scratch
	protocol. Expects a string without its delimiting quotes. Returns a new
	escaped string. 
	"""
	escaped = ''	
	for c in msg: 
	    escaped += c
	    if c == '"':
		escaped += '"'
	return escaped

    def _unescape(self, msg):
	"""
	Removes double quotes that were used to escape double quotes. Expects
	a string without its delimiting quotes, or a number. Returns a new
	unescaped string.
	"""
	if isinstance(msg, (int, float, long)):
	    return msg

	unescaped = ''
	i = 0
	while i < len(msg):
	    unescaped += msg[i]
	    if msg[i] == '"':
		i+=1
	    i+=1
	return unescaped     

    def _is_msg(self, msg):
	"""
	Returns True if message is a proper Scratch message, else return False.
	"""
	if not msg or len(msg) < self.prefix_len:
	    return False
	length = self._extract_len(msg[:self.prefix_len])
	msg_type = msg[self.prefix_len:].split(' ', 1)[0]
	if length == len(msg[self.prefix_len:]) and msg_type in self.msg_types:
	    return True
	return False

    def _parse_broadcast(self, msg):
	"""
	Given a broacast message, returns the message that was broadcast.
	"""
	# get message, remove surrounding quotes, and unescape
	return self._unescape(self._get_type(msg[self.broadcast_prefix_len:]))

    def _parse_sensorupdate(self, msg):
	"""
	Given a sensor-update message, returns the sensors/variables that were
	updated as a dict that maps sensors/variables to their updated values.
	"""
	update = msg[self.sensorupdate_prefix_len:]
	parsed = [] # each element is either a sensor (key) or a sensor value
	curr_seg = '' # current segment (i.e. key or value) being formed
	numq = 0 # number of double quotes in current segment
	for seg in update.split(' ')[:-1]: # last char in update is a space
	    numq += seg.count('"')
	    curr_seg += seg
	    # even number of quotes means we've finished parsing a segment
	    if numq % 2 == 0: 
		parsed.append(curr_seg)
		curr_seg = ''
		numq = 0
	    else: # segment has a space inside, so add back it in
		curr_seg += ' '
	unescaped = [self._unescape(self._get_type(x)) for x in parsed]
	# combine into a dict using iterators (both elements in the list
	# inputted to izip have a reference to the same iterator). even 
	# elements are keys, odd are values
	return dict(itertools.izip(*[iter(unescaped)]*2)) 

    def _parse(self, msg):
	"""
	Parses a Scratch message and returns a tuple with the first element
	as the message type, and the second element as the message payload. The 
	payload for a 'broadcast' message is a string, and the payload for a 
	'sensor-update' message is a dict whose keys are variables, and values
	are updated variable values. Returns None if msg is not a message.
	"""
	if not self._is_msg(msg):
	    return None
	msg_type = msg[self.prefix_len:].split(' ')[0]
	if msg_type == 'broadcast':
	    return ('broadcast', self._parse_broadcast(msg))
	else:
	    return ('sensor-update', self._parse_sensorupdate(msg))

    def _write(self, data):
	"""
	Writes string data out to Scratch
	"""
	total_sent = 0
	length = len(data)
	while total_sent < length:
	    try:
		sent = self.socket.send(data[total_sent:])
	    except socket.error as (err, msg):
		self.connected = False
		raise ScratchError("[Errno %d] %s" % (err, msg))
	    if sent == 0:
		self.connected = False
		raise ScratchConnectionError("Connection broken")
	    total_sent += sent

    def _send(self, data):
	"""
	Sends a message to Scratch
	"""
	self._write(self._pack(data))

    def _read(self, size):
	"""
	Reads size number of bytes from Scratch and returns data as a string
	"""
	data = ''
	while len(data) < size:
	    try:
		chunk = self.socket.recv(size-len(data))
	    except socket.error as (err, msg):
		self.connected = False
		raise ScratchError("[Errno %d] %s" % (err, msg))
	    if chunk == '':
		self.connected = False
		raise ScratchConnectionError("Connection broken")
	    data += chunk
	return data

    def _recv(self):
	"""
	Receives and returns a message from Scratch
	"""
	prefix = self._read(self.prefix_len)
	msg = self._read(self._extract_len(prefix))
	return prefix + msg

    def connect(self):
	"""
	Connects to Scratch.
	"""
	self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	try:
	    self.socket.connect((self.host, self.port))
	except socket.error as (err, msg):
	    self.connected = False
	    raise ScratchError("[Errno %d] %s" % (err, msg))
	self.connected = True

    def disconnect(self):
	"""
	Closes connection to Scratch
	"""
	try: # connection may already be disconnected, so catch exceptions
	    self.socket.shutdown(socket.SHUT_RDWR) # a proper disconnect
	except socket.error:
	    pass
	self.socket.close()
	self.connected = False

    def sensorupdate(self, data):
	"""
	Given a dict of sensors and values, updates those sensors with the 
	values in Scratch.
	"""
	if not isinstance(data, dict):
	    raise TypeError('Expected a dict')
	msg = 'sensor-update '
	for key in data.keys():
	    msg += '"%s" "%s" ' % (self._escape(str(key)), 
				   self._escape(str(data[key])))
	self._send(msg)

    def broadcast(self, msg):
	"""
	Broadcasts msg to Scratch. msg can be a single message or an iterable 
	(list, tuple, set, generator, etc.) of messages.
	"""
	if getattr(msg, '__iter__', False): # iterable
	    for m in msg:
		self._send('broadcast "%s"' % self._escape(str(m)))
	else: # probably a string or number
	    self._send('broadcast "%s"' % self._escape(str(msg)))

    def receive(self):
	"""
	Receives broadcasts and sensor updates from Scratch. Returns a tuple
	with the first element as the message type and the second element 
	as the message payload. broadcast messages have a string as payload, 
	and the sensor-update messages have a dict as payload. Returns None if 
	message received could not be parsed. Raises exceptions on connection
	errors.
	"""
	return self._parse(self._recv())



#!/usr/bin/env python
#
# Copyright (c) 2015 mindsensors.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date      Author      Comments
# Oct 2015  Michael     Initial Authoring

import os,sys,inspect,time,thread
import socket,fcntl,struct,ms_explorerlib    

currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0,parentdir) 
from PiStorms import PiStorms

print "running program"
psm = PiStorms()
psm.screen.termPrintln("Connect your I2C device to BAS1")
psm.screen.termPrintln("and click Explore")
psm.screen.drawButton(75, 95, width = 85, height = 40, text="Explore", display=False)
psm.screen.drawButton(175, 95, width = 60, height = 40, text="Exit", display=True)

exit = False
lastled = 0

def explore():
    time.sleep(3)
    addr = 0x00     # DO NOT change this address!!!
    i2c = ms_explorerlib.Explorer(addr)
    found = i2c.ping(0x00)  
    # Checks for connection on all I2C addresses until connection is found
    count = 0 
    while found == -1:     
        if (addr < 0xef):
            addr = addr + 1
            count = count + 1
            if (addr != 0x34 and addr != 0x35 and addr != 0x36 and addr != 0x37):
                i2c = ms_explorerlib.Explorer(addr)
                found = i2c.ping(0x00)
            if (count > 2000):
                found = 5
        else:
            addr = 0x00   
    if (found == 5):
        psm.screen.termPrintAt(5, "No Device found!")
        psm.screen.termPrintAt(6, "Click Exit to return to main menu")
    else:
        psm.screen.termPrintAt(5, "7 bit address: " + str(hex(addr/2)))      
        psm.screen.termPrintAt(6, "8 bit address: " + str(hex(addr)))         
        psm.screen.termPrintAt(7, "FW Version: " + i2c.GetFirmwareVersion()[:5])
        psm.screen.termPrintAt(8, "Vendor ID: " + i2c.GetVendorName())
        device = i2c.GetDeviceId()
        pos = device.find('\0') 
        psm.screen.termPrintAt(9, "Device ID: " + i2c.GetDeviceId()[:pos])

while(not exit):
    explorer = psm.screen.checkButton(75, 95,width=85,height=40)
    bye = psm.screen.checkButton(175, 95,width=60,height=40)
    if(explorer == True):
        psm.screen.termPrintAt(5, "")
        psm.screen.termPrintAt(6, "")
        psm.screen.termPrintAt(7, "")
        psm.screen.termPrintAt(8, "")
        psm.screen.termPrintAt(9, "")
        psm.screen.termPrintAt(5, "Searching for i2c device...")        
        psm.BAS1.activateCustomSensorI2C()
        explore()
    if(bye == True):
        psm.screen.termPrintAt(5, "Exiting to menu")
        exit = True
    time.sleep(.1)


#!/usr/bin/env python
#
# Copyright (c) 2015 mindsensors.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date      Author      Comments
#  August 20, 2015  Andrew Miller     Initial Authoring 

from PiStorms import PiStorms

#Demo Code for the PiStorms and Raspberry Pi


#initial setup code
import os,sys,inspect,time,thread, random
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
#sys.path.insert(0,parentdir) 
from PiStorms  import PiStorms 

#starts an instance of PiStorms 
psm = PiStorms ()

#exit variable will be used later to exit the program and return to PiStorms 
exit = False

#clears the screen of any unwanted text by displaying a white rectangle
psm.screen.fillRect(0, 0, 320, 240)

#displays Sam's greeting of "Hello, I am Sam"
psm.screen.drawAutoText("Hello", 80, 30, fill=(255, 0, 0), size = 70)
psm.screen.drawAutoText("I am Sam", 70, 140, fill=(255, 0, 0), size = 45)
psm.BBM2.runSecs(1,20,True);
psm.BAM2.runSecs(1,20,True);


#main loop
while(not exit):
    
    if(psm.BBS2.distanceUSEV3() < 200): #if ultrasonic sensor reading is <200 (~6")
        
        #Quickly move arms backward 130 degrees and brake and hold them there
        psm.BBM2.runDegs(-(130), brakeOnCompletion = True, holdOnCompletion = False)
        psm.BAM2.runDegs(-(130), brakeOnCompletion = True, holdOnCompletion = False)
        
        #display scared emoticon
        psm.screen.fillBmp(30, 0, 240, 240, path = currentdir+'/'+"faceScared2.png")
        
        #release motors from brake and hold
        psm.BBM2.float()
        psm.BAM2.float()
        
        #wait one second
        time.sleep(1)
        
        #clear screen of scared emoticon
        psm.screen.fillRect(0, 0, 320, 240)
        
        #Display "sorry you scared me" for 2 seconds, then clear screen
        psm.screen.drawAutoText("Sorry,", 75, 30, fill=(0, 0, 0), size = 70)
        psm.screen.drawAutoText("you scared me", 15, 140, fill=(0, 0, 0), size = 45)
        time.sleep(2)
        psm.screen.fillRect(0, 0, 320, 240)
        
        #lower arms back to starting position and brake and hold them
        #only needs 120 degrees so they won't slam into the table
        #gravity+intertia will lower them the remaining 10 degrees
        psm.BBM2.runSecs(1,20,True);
        psm.BAM2.runSecs(1,20,True);
        
        
        
        #redraw "Hello I am Sam"        
        psm.screen.drawAutoText("Hello", 80, 30, fill=(255, 0, 0), size = 70)
        psm.screen.drawAutoText("I am Sam", 70, 140, fill=(255, 0, 0), size = 45)
        
        
    if(psm.isKeyPressed() == True): # if the GO button is pressed
        psm.screen.clearScreen()
        psm.screen.termPrintln("") 
        psm.screen.termPrintln("Exiting to menu")
        time.sleep(0.5) 
        exit = True 


#!/usr/bin/env python
#
# Copyright (c) 2015 mindsensors.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date      Author      Comments

import os,sys,inspect,time,thread
import socket,fcntl,struct


def get_ip_address(ifname):
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    return socket.inet_ntoa(fcntl.ioctl(
        s.fileno(),
        0x8915,  # SIOCGIFADDR
        struct.pack('256s', ifname[:15])
    )[20:24])
    
    

from PiStorms import PiStorms
print "running psm-info.py"
psm = PiStorms()

print " Version : "+ str(psm.GetFirmwareVersion() )[:5]
print " Vendor  : "+ str(psm.GetVendorName() )
print " Device : "+ str(psm.GetDeviceId() )
print " HostName :     "   + socket.gethostname()
   
try:
    print " eth0 :     "   + get_ip_address('eth0')
except:
    print " eth0 : not present"
try:    
    print " wlan0 :    "+ get_ip_address('wlan0')
except:
    print " wlan0 : not present"
   
